# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'viewer0D_GUI.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(598, 336)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.splitter = QtWidgets.QSplitter(Form)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.verticalLayoutWidget = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.Nhistory_sb = QtWidgets.QSpinBox(self.verticalLayoutWidget)
        self.Nhistory_sb.setReadOnly(False)
        self.Nhistory_sb.setMinimum(1)
        self.Nhistory_sb.setMaximum(1000000)
        self.Nhistory_sb.setSingleStep(100)
        self.Nhistory_sb.setProperty("value", 200)
        self.Nhistory_sb.setObjectName("Nhistory_sb")
        self.horizontalLayout_2.addWidget(self.Nhistory_sb)
        self.clear_pb = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.clear_pb.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/clear2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.clear_pb.setIcon(icon)
        self.clear_pb.setCheckable(False)
        self.clear_pb.setObjectName("clear_pb")
        self.horizontalLayout_2.addWidget(self.clear_pb)
        self.show_datalist_pb = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.show_datalist_pb.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/ChnNum.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.show_datalist_pb.setIcon(icon1)
        self.show_datalist_pb.setCheckable(True)
        self.show_datalist_pb.setObjectName("show_datalist_pb")
        self.horizontalLayout_2.addWidget(self.show_datalist_pb)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.Graph1D = PlotWidget(self.verticalLayoutWidget)
        self.Graph1D.setObjectName("Graph1D")
        self.verticalLayout.addWidget(self.Graph1D)
        self.values_list = QtWidgets.QListWidget(self.splitter)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.values_list.setFont(font)
        self.values_list.setObjectName("values_list")
        self.gridLayout.addWidget(self.splitter, 0, 0, 1, 1)
        self.StatusBarLayout = QtWidgets.QHBoxLayout()
        self.StatusBarLayout.setObjectName("StatusBarLayout")
        self.gridLayout.addLayout(self.StatusBarLayout, 1, 0, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.Nhistory_sb.setToolTip(_translate("Form", "N samples in plot"))
        self.clear_pb.setToolTip(_translate("Form", "Clear plot"))
        self.show_datalist_pb.setToolTip(_translate("Form", "Show current data in a list"))

from pyqtgraph import PlotWidget
from pymodaq.QtDesigner_Ressources import QtDesigner_ressources_rc

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




from PyQt5 import QtGui, QtWidgets
from PyQt5.QtCore import Qt,QObject, pyqtSlot, QThread, pyqtSignal, QLocale, QDateTime, QSize
from PyQt5.QtWidgets import QAbstractItemView
import sys
from PyMoDAQ.DAQ_Utils.plotting.select_item_tolist_GUI import Ui_Form
from PyMoDAQ.DAQ_Utils.DAQ_enums import Items_Lockin_SR830


#class Select_item_tolist(Ui_Form,QObject):
#    updated_items=pyqtSignal(list)
#    def __init__(self,parent,items_list=[],preset_items=[]):
#        QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
#        super(Select_item_tolist,self).__init__()
        
#        self.ui=Ui_Form()
#        self.ui.setupUi(parent)
#        self.ui.parent=parent

#        self.ui.item_list=custom_list_widget(self)
#        self.ui.horizontalLayout.addWidget(self.ui.item_list)
#        self.item_list=items_list
#        self.preset_items=preset_items

#        self.ui.items_cb.addItems(items_list)
#        self.ui.all_items_list.addItems(items_list)
#        self.ui.item_list.addItems(preset_items)

#        self.ui.add_item_pb.clicked.connect(self.add_item)
#        self.ui.remove_item_pb.clicked.connect(self.remove_item)
#        self.items=[item for item in preset_items]
  
#    def populate_items_cb(self,items):
#        self.ui.items_cb.clear()
#        self.ui.items.addItems(items)
#        self.ui.all_items_list.clear()
#        self.ui.all_items_list.addItems(items)

#    def ini_items(self,items_tot):
#        self.item_list=items_tot['all_items']
#        self.preset_items=items_tot['preset']
#        self.ui.items_cb.clear()
#        self.ui.item_list.clear()
#        self.ui.all_items_list.clear()
#        self.ui.items_cb.addItems(self.item_list)
#        self.ui.all_items_list.addItems(self.item_list)
#        self.ui.item_list.addItems(self.preset_items)
#        self.items=self.all_items_func()
            
#    def get_value(self):
#        return dict(all_items=self.item_list,preset=self.items)

#    def add_item(self):
        
#        items_str=self.all_items_func()
#        if not(self.ui.items_cb.currentText() in items_str):
#            self.ui.item_list.addItem(self.ui.items_cb.currentText())
#        self.items=self.all_items_func()
        

#    def remove_item(self):
#        self.ui.item_list.takeItem(self.ui.item_list.currentRow())
#        self.items=self.all_items_func()
        

#    def all_items_func(self):
#        Nitems=self.ui.item_list.count()
#        items_str=[]
#        for ind in range (Nitems):
#            items_str.append(self.ui.item_list.item(ind).text())
#        self.updated_items.emit(items_str)
#        return items_str

class custom_list_widget(QtWidgets.QListWidget):
     def __init__(self,parent,items_list=[],preset_items=[]):
        QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
        super(custom_list_widget,self).__init__()
        self.parent=parent
        self.setDragEnabled(True)
        self.setDragDropMode(QAbstractItemView.DragDrop)
        self.setToolTip('list of scan items')

     def dropEvent(self,event):
        item_txt=event.source().currentItem().text()
        index_from=event.source().currentIndex().row()
        index_to=self.indexAt(event.pos()).row()
        
        if event.source()==self:
            if index_to>index_from:
                index_to=index_to
            if index_to<0:
                index_to=0
            if index_from==index_to:
                pass
            else:
                item=self.takeItem(index_from)
                self.insertItem(index_to,item.text())
            
        else:
           if item_txt not in self.parent.items:
                index=0
                index=self.indexAt(event.pos()).row()+1
                self.insertItem(index,item_txt)
                
        self.parent.items=self.parent.all_items_func()


#%%
class Select_item_tolist_simpler(QtWidgets.QWidget):
    updated_items=pyqtSignal(list)
    def __init__(self,items_list=[],preset_items=[]):
        QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
        super(Select_item_tolist_simpler,self).__init__()
        self.initUI()

        self.all_items=items_list
        self.preset_items=preset_items

        self.all_items_list.addItems(items_list)
        self.current_items_list.addItems(preset_items)
        self.current_items_list.itemDoubleClicked[QtWidgets.QListWidgetItem].connect(self.remove_item)
#        self.add_item_pb.clicked.connect(self.add_item)
#        self.remove_item_pb.clicked.connect(self.remove_item)
        self.items=[item for item in preset_items]


    def initUI(self):
        
        #self.setGeometry(300, 300, 300, 220)
        self.setWindowTitle('test')
        
        self.hor_layout_label=QtWidgets.QHBoxLayout()
        self.label1=QtWidgets.QLabel('Select from:')
        self.label2=QtWidgets.QLabel('Selected:')
        self.hor_layout_label.addWidget(self.label1)
        self.hor_layout_label.addWidget(self.label2)
        
        self.hor_layout=QtWidgets.QHBoxLayout()
        self.all_items_list=QtWidgets.QListWidget()
        self.all_items_list.setMaximumHeight(100)
        self.all_items_list.setDragEnabled(True)
        self.all_items_list.setDragDropMode(QtWidgets.QAbstractItemView.DragOnly)
        self.current_items_list=custom_list_widget(self)
        self.current_items_list.setMaximumHeight(100)
        self.hor_layout.addWidget(self.all_items_list)
        self.hor_layout.addWidget(self.current_items_list)
        
        self.vbox = QtWidgets.QVBoxLayout()
        self.vbox.addLayout(self.hor_layout_label)
        self.vbox.addLayout(self.hor_layout)
        
        self.setLayout(self.vbox)
        
  
    def populate_items_cb(self,items):
        self.ui.items_cb.clear()
        self.ui.items.addItems(items)
        self.ui.all_items_list.clear()
        self.ui.all_items_list.addItems(items)

    def ini_items(self,items_tot):
        self.all_items=items_tot['all_items']
        self.preset_items=items_tot['preset']
        self.all_items_list.clear()
        self.current_items_list.clear()
        self.all_items_list.addItems(self.all_items)
        self.current_items_list.addItems(self.preset_items)
        self.items=self.all_items_func()
            
    def get_value(self):
        return dict(all_items=self.all_items,preset=self.items)

    @pyqtSlot(QtWidgets.QListWidgetItem)
    def remove_item(self,item):
        self.current_items_list.takeItem(self.current_items_list.row(item))

        self.items=self.all_items_func()
        

    def all_items_func(self):
        Nitems=self.current_items_list.count()
        items_str=[]
        for ind in range (Nitems):
            items_str.append(self.current_items_list.item(ind).text())
        self.updated_items.emit(items_str)
        
        return items_str







if __name__ == '__main__':
    items=Items_Lockin_SR830.names(Items_Lockin_SR830);
    preset_items=items[2:4];
                      
    app = QtWidgets.QApplication(sys.argv);
    ex=Select_item_tolist_simpler(items,preset_items)
    ex.show()                     
#    prog = Select_item_tolist(Form,items,preset_items);                              
#    Form = QtWidgets.QWidget();
#    
#    Form.show();
    sys.exit(app.exec_())




import distutils.dir_util
from distutils.command import build
import os, sys, re
try:
    import setuptools
    from setuptools import setup, find_packages
    from setuptools.command import install
except ImportError:
    sys.stderr.write("Warning: could not import setuptools; falling back to distutils.\n")
    from distutils.core import setup
    from distutils.command import install

from pymodaq.version import get_version

with open('README.rst') as fd:
    long_description = fd.read()

setupOpts = dict(
    name='pymodaq',
    description='Modular Data Acquisition with Python',
    long_description=long_description,
    license='MIT',
    url='http://pymodaq.cnrs.fr',
    author='Sébastien Weber',
    author_email='sebastien.weber@cemes.fr',
    classifiers = [
        "Programming Language :: Python :: 3",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Other Environment",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Human Machine Interfaces",
        "Topic :: Scientific/Engineering :: Visualization",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: User Interfaces",
        ],)

def listAllPackages(pkgroot):
    path = os.getcwd()
    n = len(path.split(os.path.sep))
    subdirs = [i[0].split(os.path.sep)[n:] for i in os.walk(os.path.join(path, pkgroot)) if '__init__.py' in i[2]]
    return ['.'.join(p) for p in subdirs]


allPackages = (listAllPackages(pkgroot='pymodaq')) #+
               #['pyqtgraph.'+x for x in helpers.listAllPackages(pkgroot='examples')])



def get_packages():

    packages=find_packages()
    for pkg in packages:
        if 'hardware.' in pkg:
            packages.pop(packages.index(pkg))
    return packages

allPackages = get_packages()



setup(
    version=get_version(),
     #cmdclass={'build': Build,},
    #           'install': Install,
    #           'deb': helpers.DebCommand,
    #           'test': helpers.TestCommand,
    #           'debug': helpers.DebugCommand,
    #           'mergetest': helpers.MergeTestCommand,
    #           'style': helpers.StyleCommand},
    packages=allPackages,
    #package_dir={'examples': 'examples'},  ## install examples along with the rest of the source
    package_data={},
    entry_points={'console_scripts':['pymodaq_scan=pymodaq.pymodaq_exec:scan',
                                     'pymodaq_move=pymodaq.pymodaq_exec:move',
                                    'pymodaq_viewer=pymodaq.pymodaq_exec:viewer',
                                    'pymodaq_h5browser=pymodaq.pymodaq_exec:h5browser'
                                     ],
                   'gui_scripts': ['pymodaq_scan=pymodaq.pymodaq_exec:scan',
                                    'pymodaq_move=pymodaq.pymodaq_exec:move',
                                    'pymodaq_viewer=pymodaq.pymodaq_exec:viewer',
                                    'pymodaq_h5browser=pymodaq.pymodaq_exec:h5browser'
                                    ]
                  },
    install_requires = [
        'numpy',
        'scipy',
        'pyqtgraph==0.10',
        'bitstring',
        'easydict',
        'pyqt5',
        'numba',
        'tables',
        'pymodaq_plugins>=1.2.0',
        'pymodaq_pid_models',
        'dask',
		'simple_pid',
        ],
    include_package_data=True,
    **setupOpts
)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'DAQ_Navigation_Visu_GUI.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(878, 848)
        MainWindow.setBaseSize(QtCore.QSize(1000, 800))
        MainWindow.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.splitter_7 = QtWidgets.QSplitter(self.centralwidget)
        self.splitter_7.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_7.setObjectName("splitter_7")
        self.tabWidget_2 = QtWidgets.QTabWidget(self.splitter_7)
        self.tabWidget_2.setObjectName("tabWidget_2")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.plotting_layout = QtWidgets.QVBoxLayout()
        self.plotting_layout.setObjectName("plotting_layout")
        self.gridLayout_9.addLayout(self.plotting_layout, 0, 0, 1, 1)
        self.tabWidget_2.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.gridLayout_17 = QtWidgets.QGridLayout(self.tab_3)
        self.gridLayout_17.setObjectName("gridLayout_17")
        self.splitter_5 = QtWidgets.QSplitter(self.tab_3)
        self.splitter_5.setOrientation(QtCore.Qt.Vertical)
        self.splitter_5.setObjectName("splitter_5")
        self.splitter_6 = QtWidgets.QSplitter(self.splitter_5)
        self.splitter_6.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_6.setObjectName("splitter_6")
        self.verticalLayoutWidget_3 = QtWidgets.QWidget(self.splitter_6)
        self.verticalLayoutWidget_3.setObjectName("verticalLayoutWidget_3")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_3)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.Tree = QtWidgets.QTreeWidget(self.verticalLayoutWidget_3)
        self.Tree.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.Tree.setObjectName("Tree")
        self.Tree.headerItem().setText(0, "1")
        self.verticalLayout_8.addWidget(self.Tree)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.Open_Tree = QtWidgets.QPushButton(self.verticalLayoutWidget_3)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/tree.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.Open_Tree.setIcon(icon)
        self.Open_Tree.setObjectName("Open_Tree")
        self.horizontalLayout_2.addWidget(self.Open_Tree)
        self.Open_Tree_Selected = QtWidgets.QPushButton(self.verticalLayoutWidget_3)
        self.Open_Tree_Selected.setIcon(icon)
        self.Open_Tree_Selected.setObjectName("Open_Tree_Selected")
        self.horizontalLayout_2.addWidget(self.Open_Tree_Selected)
        self.Close_Tree = QtWidgets.QPushButton(self.verticalLayoutWidget_3)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/CollapseAll.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.Close_Tree.setIcon(icon1)
        self.Close_Tree.setObjectName("Close_Tree")
        self.horizontalLayout_2.addWidget(self.Close_Tree)
        self.verticalLayout_8.addLayout(self.horizontalLayout_2)
        self.verticalLayoutWidget_4 = QtWidgets.QWidget(self.splitter_6)
        self.verticalLayoutWidget_4.setObjectName("verticalLayoutWidget_4")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_4)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.Data1D_graph = PlotWidget(self.splitter_5)
        self.Data1D_graph.setObjectName("Data1D_graph")
        self.gridLayout_17.addWidget(self.splitter_5, 0, 0, 1, 1)
        self.tabWidget_2.addTab(self.tab_3, "")
        self.splitter_3 = QtWidgets.QSplitter(self.splitter_7)
        self.splitter_3.setMinimumSize(QtCore.QSize(300, 0))
        self.splitter_3.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.splitter_3.setOrientation(QtCore.Qt.Vertical)
        self.splitter_3.setObjectName("splitter_3")
        self.tabWidget = QtWidgets.QTabWidget(self.splitter_3)
        self.tabWidget.setBaseSize(QtCore.QSize(0, 0))
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.splitter_4 = QtWidgets.QSplitter(self.tab)
        self.splitter_4.setOrientation(QtCore.Qt.Vertical)
        self.splitter_4.setObjectName("splitter_4")
        self.layoutWidget = QtWidgets.QWidget(self.splitter_4)
        self.layoutWidget.setObjectName("layoutWidget")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.layoutWidget)
        self.gridLayout_7.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.save_process_pb = QtWidgets.QPushButton(self.layoutWidget)
        self.save_process_pb.setObjectName("save_process_pb")
        self.gridLayout_7.addWidget(self.save_process_pb, 7, 2, 1, 1)
        self.process_data_pb = QtWidgets.QPushButton(self.layoutWidget)
        self.process_data_pb.setObjectName("process_data_pb")
        self.gridLayout_7.addWidget(self.process_data_pb, 7, 0, 1, 2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.result_edit_blue = QtWidgets.QLineEdit(self.layoutWidget)
        self.result_edit_blue.setEnabled(True)
        self.result_edit_blue.setReadOnly(True)
        self.result_edit_blue.setObjectName("result_edit_blue")
        self.horizontalLayout_4.addWidget(self.result_edit_blue)
        self.result_edit_green = QtWidgets.QLineEdit(self.layoutWidget)
        self.result_edit_green.setEnabled(True)
        self.result_edit_green.setReadOnly(True)
        self.result_edit_green.setObjectName("result_edit_green")
        self.horizontalLayout_4.addWidget(self.result_edit_green)
        self.result_edit_red = QtWidgets.QLineEdit(self.layoutWidget)
        self.result_edit_red.setEnabled(True)
        self.result_edit_red.setReadOnly(True)
        self.result_edit_red.setObjectName("result_edit_red")
        self.horizontalLayout_4.addWidget(self.result_edit_red)
        self.gridLayout_7.addLayout(self.horizontalLayout_4, 6, 0, 1, 3)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.result_blue_label = QtWidgets.QPushButton(self.layoutWidget)
        self.result_blue_label.setEnabled(True)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/b_icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.result_blue_label.setIcon(icon2)
        self.result_blue_label.setFlat(True)
        self.result_blue_label.setObjectName("result_blue_label")
        self.horizontalLayout_5.addWidget(self.result_blue_label)
        self.result_green_label = QtWidgets.QPushButton(self.layoutWidget)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/g_icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.result_green_label.setIcon(icon3)
        self.result_green_label.setFlat(True)
        self.result_green_label.setObjectName("result_green_label")
        self.horizontalLayout_5.addWidget(self.result_green_label)
        self.result_red_label = QtWidgets.QPushButton(self.layoutWidget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/r_icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.result_red_label.setIcon(icon4)
        self.result_red_label.setFlat(True)
        self.result_red_label.setObjectName("result_red_label")
        self.horizontalLayout_5.addWidget(self.result_red_label)
        self.gridLayout_7.addLayout(self.horizontalLayout_5, 5, 0, 1, 3)
        self.colors_combo = QtWidgets.QComboBox(self.layoutWidget)
        self.colors_combo.setObjectName("colors_combo")
        self.gridLayout_7.addWidget(self.colors_combo, 0, 0, 1, 1)
        self.merge_spectra_cb = QtWidgets.QCheckBox(self.layoutWidget)
        self.merge_spectra_cb.setObjectName("merge_spectra_cb")
        self.gridLayout_7.addWidget(self.merge_spectra_cb, 0, 1, 1, 2)
        self.label_8 = QtWidgets.QLabel(self.layoutWidget)
        self.label_8.setObjectName("label_8")
        self.gridLayout_7.addWidget(self.label_8, 9, 0, 1, 3)
        self.comments_edit = QtWidgets.QTextEdit(self.layoutWidget)
        self.comments_edit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.comments_edit.setObjectName("comments_edit")
        self.gridLayout_7.addWidget(self.comments_edit, 10, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem, 11, 0, 1, 3)
        self.toolBox_Analysis = QtWidgets.QToolBox(self.splitter_4)
        self.toolBox_Analysis.setFrameShape(QtWidgets.QFrame.Panel)
        self.toolBox_Analysis.setFrameShadow(QtWidgets.QFrame.Raised)
        self.toolBox_Analysis.setObjectName("toolBox_Analysis")
        self.page_measurement = QtWidgets.QWidget()
        self.page_measurement.setGeometry(QtCore.QRect(0, 0, 293, 214))
        self.page_measurement.setObjectName("page_measurement")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.page_measurement)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_7 = QtWidgets.QLabel(self.page_measurement)
        self.label_7.setAlignment(QtCore.Qt.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 5, 0, 1, 1)
        self.operation_edit = QtWidgets.QTextEdit(self.page_measurement)
        self.operation_edit.setMaximumSize(QtCore.QSize(16777215, 55))
        self.operation_edit.setObjectName("operation_edit")
        self.gridLayout.addWidget(self.operation_edit, 6, 1, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.page_measurement)
        self.label_20.setObjectName("label_20")
        self.gridLayout.addWidget(self.label_20, 6, 0, 1, 1)
        self.measure_subtype_combo = QtWidgets.QComboBox(self.page_measurement)
        self.measure_subtype_combo.setObjectName("measure_subtype_combo")
        self.measure_subtype_combo.addItem("")
        self.measure_subtype_combo.addItem("")
        self.measure_subtype_combo.addItem("")
        self.gridLayout.addWidget(self.measure_subtype_combo, 2, 1, 1, 1)
        self.variables_edit = QtWidgets.QLineEdit(self.page_measurement)
        self.variables_edit.setText("")
        self.variables_edit.setReadOnly(True)
        self.variables_edit.setObjectName("variables_edit")
        self.gridLayout.addWidget(self.variables_edit, 3, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.page_measurement)
        self.label_9.setAlignment(QtCore.Qt.AlignCenter)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 0, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.page_measurement)
        self.label_11.setAlignment(QtCore.Qt.AlignCenter)
        self.label_11.setObjectName("label_11")
        self.gridLayout.addWidget(self.label_11, 2, 0, 1, 1)
        self.measure_type_combo = QtWidgets.QComboBox(self.page_measurement)
        self.measure_type_combo.setObjectName("measure_type_combo")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.gridLayout.addWidget(self.measure_type_combo, 0, 1, 1, 1)
        self.formula_edit = QtWidgets.QPlainTextEdit(self.page_measurement)
        self.formula_edit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.formula_edit.setPlainText("")
        self.formula_edit.setTextInteractionFlags(QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.formula_edit.setObjectName("formula_edit")
        self.gridLayout.addWidget(self.formula_edit, 5, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.page_measurement)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)
        self.do_operation_cb = QtWidgets.QCheckBox(self.page_measurement)
        self.do_operation_cb.setObjectName("do_operation_cb")
        self.gridLayout.addWidget(self.do_operation_cb, 7, 0, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)
        self.toolBox_Analysis.addItem(self.page_measurement, "")
        self.page_bkg = QtWidgets.QWidget()
        self.page_bkg.setGeometry(QtCore.QRect(0, 0, 480, 115))
        self.page_bkg.setObjectName("page_bkg")
        self.gridLayout_14 = QtWidgets.QGridLayout(self.page_bkg)
        self.gridLayout_14.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_14.setObjectName("gridLayout_14")
        self.gridLayout_13 = QtWidgets.QGridLayout()
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.apply_bkg_check = QtWidgets.QCheckBox(self.page_bkg)
        self.apply_bkg_check.setObjectName("apply_bkg_check")
        self.gridLayout_13.addWidget(self.apply_bkg_check, 1, 0, 1, 1)
        self.set_whole_as_bkg_cb = QtWidgets.QCheckBox(self.page_bkg)
        self.set_whole_as_bkg_cb.setObjectName("set_whole_as_bkg_cb")
        self.gridLayout_13.addWidget(self.set_whole_as_bkg_cb, 0, 0, 1, 2)
        self.bkg_type = QtWidgets.QComboBox(self.page_bkg)
        self.bkg_type.setObjectName("bkg_type")
        self.bkg_type.addItem("")
        self.bkg_type.addItem("")
        self.gridLayout_13.addWidget(self.bkg_type, 1, 1, 1, 1)
        self.background_edit = QtWidgets.QLineEdit(self.page_bkg)
        self.background_edit.setReadOnly(True)
        self.background_edit.setObjectName("background_edit")
        self.gridLayout_13.addWidget(self.background_edit, 2, 0, 1, 3)
        self.do_zero_cb = QtWidgets.QCheckBox(self.page_bkg)
        self.do_zero_cb.setObjectName("do_zero_cb")
        self.gridLayout_13.addWidget(self.do_zero_cb, 1, 2, 1, 1)
        self.set_as_bkg_pb = QtWidgets.QPushButton(self.page_bkg)
        self.set_as_bkg_pb.setObjectName("set_as_bkg_pb")
        self.gridLayout_13.addWidget(self.set_as_bkg_pb, 0, 2, 1, 1)
        self.gridLayout_14.addLayout(self.gridLayout_13, 0, 0, 1, 1)
        self.toolBox_Analysis.addItem(self.page_bkg, "")
        self.page_spikes = QtWidgets.QWidget()
        self.page_spikes.setGeometry(QtCore.QRect(0, 0, 202, 123))
        self.page_spikes.setObjectName("page_spikes")
        self.gridLayout_16 = QtWidgets.QGridLayout(self.page_spikes)
        self.gridLayout_16.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_16.setHorizontalSpacing(6)
        self.gridLayout_16.setObjectName("gridLayout_16")
        self.gridLayout_15 = QtWidgets.QGridLayout()
        self.gridLayout_15.setObjectName("gridLayout_15")
        self.show_despiked_cb = QtWidgets.QCheckBox(self.page_spikes)
        self.show_despiked_cb.setObjectName("show_despiked_cb")
        self.gridLayout_15.addWidget(self.show_despiked_cb, 0, 1, 1, 1)
        self.threshold_sb = QtWidgets.QDoubleSpinBox(self.page_spikes)
        self.threshold_sb.setMinimum(-100000.0)
        self.threshold_sb.setMaximum(100000.0)
        self.threshold_sb.setSingleStep(0.01)
        self.threshold_sb.setProperty("value", 0.8)
        self.threshold_sb.setObjectName("threshold_sb")
        self.gridLayout_15.addWidget(self.threshold_sb, 1, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.page_spikes)
        self.label_12.setObjectName("label_12")
        self.gridLayout_15.addWidget(self.label_12, 1, 0, 1, 1)
        self.remove_spikes_cb = QtWidgets.QCheckBox(self.page_spikes)
        self.remove_spikes_cb.setObjectName("remove_spikes_cb")
        self.gridLayout_15.addWidget(self.remove_spikes_cb, 0, 0, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.page_spikes)
        self.label_16.setObjectName("label_16")
        self.gridLayout_15.addWidget(self.label_16, 2, 0, 1, 1)
        self.despiked_width_spikes_sb = QtWidgets.QSpinBox(self.page_spikes)
        self.despiked_width_spikes_sb.setMinimum(1)
        self.despiked_width_spikes_sb.setMaximum(1000)
        self.despiked_width_spikes_sb.setProperty("value", 2)
        self.despiked_width_spikes_sb.setObjectName("despiked_width_spikes_sb")
        self.gridLayout_15.addWidget(self.despiked_width_spikes_sb, 2, 1, 1, 1)
        self.despiked_width_average_sb = QtWidgets.QSpinBox(self.page_spikes)
        self.despiked_width_average_sb.setMinimum(1)
        self.despiked_width_average_sb.setMaximum(1000)
        self.despiked_width_average_sb.setProperty("value", 6)
        self.despiked_width_average_sb.setObjectName("despiked_width_average_sb")
        self.gridLayout_15.addWidget(self.despiked_width_average_sb, 4, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.page_spikes)
        self.label_17.setObjectName("label_17")
        self.gridLayout_15.addWidget(self.label_17, 4, 0, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.page_spikes)
        self.label_18.setObjectName("label_18")
        self.gridLayout_15.addWidget(self.label_18, 5, 0, 1, 1)
        self.despiked_Nloop_sb = QtWidgets.QSpinBox(self.page_spikes)
        self.despiked_Nloop_sb.setMinimum(1)
        self.despiked_Nloop_sb.setMaximum(1000)
        self.despiked_Nloop_sb.setProperty("value", 2)
        self.despiked_Nloop_sb.setObjectName("despiked_Nloop_sb")
        self.gridLayout_15.addWidget(self.despiked_Nloop_sb, 5, 1, 1, 1)
        self.gridLayout_16.addLayout(self.gridLayout_15, 0, 0, 1, 1)
        self.toolBox_Analysis.addItem(self.page_spikes, "")
        self.gridLayout_8.addWidget(self.splitter_4, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_TreeProperties = QtWidgets.QWidget()
        self.tab_TreeProperties.setObjectName("tab_TreeProperties")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.tab_TreeProperties)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.table_properties = QtWidgets.QTableWidget(self.tab_TreeProperties)
        self.table_properties.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.table_properties.setAlternatingRowColors(True)
        self.table_properties.setWordWrap(False)
        self.table_properties.setObjectName("table_properties")
        self.table_properties.setColumnCount(0)
        self.table_properties.setRowCount(0)
        self.gridLayout_10.addWidget(self.table_properties, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_TreeProperties, "")
        self.tab_plotting = QtWidgets.QWidget()
        self.tab_plotting.setObjectName("tab_plotting")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.tab_plotting)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_15 = QtWidgets.QLabel(self.tab_plotting)
        self.label_15.setObjectName("label_15")
        self.gridLayout_4.addWidget(self.label_15, 10, 0, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.tab_plotting)
        self.label_14.setObjectName("label_14")
        self.gridLayout_4.addWidget(self.label_14, 9, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.tab_plotting)
        self.label_10.setObjectName("label_10")
        self.gridLayout_4.addWidget(self.label_10, 5, 0, 2, 1)
        self.aspect_combo = QtWidgets.QComboBox(self.tab_plotting)
        self.aspect_combo.setObjectName("aspect_combo")
        self.aspect_combo.addItem("")
        self.aspect_combo.addItem("")
        self.gridLayout_4.addWidget(self.aspect_combo, 5, 1, 2, 1)
        self.plot_2D_pb = QtWidgets.QPushButton(self.tab_plotting)
        self.plot_2D_pb.setObjectName("plot_2D_pb")
        self.gridLayout_4.addWidget(self.plot_2D_pb, 0, 0, 2, 1)
        self.plot_1D_pb = QtWidgets.QPushButton(self.tab_plotting)
        self.plot_1D_pb.setObjectName("plot_1D_pb")
        self.gridLayout_4.addWidget(self.plot_1D_pb, 0, 1, 2, 1)
        self.cmap_combo = QtWidgets.QComboBox(self.tab_plotting)
        self.cmap_combo.setObjectName("cmap_combo")
        self.gridLayout_4.addWidget(self.cmap_combo, 3, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem1, 0, 2, 13, 2)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem2, 13, 0, 1, 4)
        self.label = QtWidgets.QLabel(self.tab_plotting)
        self.label.setObjectName("label")
        self.gridLayout_4.addWidget(self.label, 3, 0, 1, 1)
        self.cmap_graphicsView = QtWidgets.QGraphicsView(self.tab_plotting)
        self.cmap_graphicsView.setMaximumSize(QtCore.QSize(2000, 25))
        self.cmap_graphicsView.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.cmap_graphicsView.setObjectName("cmap_graphicsView")
        self.gridLayout_4.addWidget(self.cmap_graphicsView, 4, 0, 1, 2)
        self.colorbar_cb = QtWidgets.QCheckBox(self.tab_plotting)
        self.colorbar_cb.setObjectName("colorbar_cb")
        self.gridLayout_4.addWidget(self.colorbar_cb, 7, 0, 1, 1)
        self.update_plot_pb = QtWidgets.QPushButton(self.tab_plotting)
        self.update_plot_pb.setObjectName("update_plot_pb")
        self.gridLayout_4.addWidget(self.update_plot_pb, 7, 1, 1, 1)
        self.save_plotting_pb = QtWidgets.QPushButton(self.tab_plotting)
        self.save_plotting_pb.setObjectName("save_plotting_pb")
        self.gridLayout_4.addWidget(self.save_plotting_pb, 12, 0, 1, 2)
        self.label_13 = QtWidgets.QLabel(self.tab_plotting)
        self.label_13.setObjectName("label_13")
        self.gridLayout_4.addWidget(self.label_13, 8, 0, 1, 1)
        self.use_labels_cb = QtWidgets.QCheckBox(self.tab_plotting)
        self.use_labels_cb.setChecked(True)
        self.use_labels_cb.setObjectName("use_labels_cb")
        self.gridLayout_4.addWidget(self.use_labels_cb, 11, 0, 1, 1)
        self.xlabel_edit = QtWidgets.QLineEdit(self.tab_plotting)
        self.xlabel_edit.setObjectName("xlabel_edit")
        self.gridLayout_4.addWidget(self.xlabel_edit, 8, 1, 1, 1)
        self.ylabel_edit = QtWidgets.QLineEdit(self.tab_plotting)
        self.ylabel_edit.setObjectName("ylabel_edit")
        self.gridLayout_4.addWidget(self.ylabel_edit, 9, 1, 1, 1)
        self.title_edit = QtWidgets.QLineEdit(self.tab_plotting)
        self.title_edit.setObjectName("title_edit")
        self.gridLayout_4.addWidget(self.title_edit, 10, 1, 1, 1)
        self.gridLayout_6.addLayout(self.gridLayout_4, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_plotting, "")
        self.toolBox = QtWidgets.QToolBox(self.splitter_3)
        self.toolBox.setObjectName("toolBox")
        self.page = QtWidgets.QWidget()
        self.page.setGeometry(QtCore.QRect(0, 0, 506, 223))
        self.page.setObjectName("page")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.page)
        self.gridLayout_12.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setHorizontalSpacing(6)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_5 = QtWidgets.QLabel(self.page)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 2, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.page)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 1, 0, 1, 1)
        self.Current_Carto_edit = QtWidgets.QLineEdit(self.page)
        self.Current_Carto_edit.setEnabled(False)
        self.Current_Carto_edit.setObjectName("Current_Carto_edit")
        self.gridLayout_3.addWidget(self.Current_Carto_edit, 1, 1, 1, 1)
        self.Current_Group_edit = QtWidgets.QLineEdit(self.page)
        self.Current_Group_edit.setEnabled(False)
        self.Current_Group_edit.setObjectName("Current_Group_edit")
        self.gridLayout_3.addWidget(self.Current_Group_edit, 2, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.page)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 4, 0, 1, 1)
        self.Current_Channel_edit = QtWidgets.QPlainTextEdit(self.page)
        self.Current_Channel_edit.setEnabled(False)
        self.Current_Channel_edit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.Current_Channel_edit.setObjectName("Current_Channel_edit")
        self.gridLayout_3.addWidget(self.Current_Channel_edit, 4, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.page)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 6, 0, 2, 1)
        self.Filenamepath = QtWidgets.QPlainTextEdit(self.page)
        self.Filenamepath.setMinimumSize(QtCore.QSize(0, 0))
        self.Filenamepath.setMaximumSize(QtCore.QSize(16777215, 50))
        self.Filenamepath.setObjectName("Filenamepath")
        self.gridLayout_3.addWidget(self.Filenamepath, 6, 1, 2, 1)
        self.gridLayout_12.addLayout(self.gridLayout_3, 0, 0, 1, 1)
        self.toolBox.addItem(self.page, "")
        self.gridLayout_5.addWidget(self.splitter_7, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 878, 21))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuLoad_Cartography_Labview = QtWidgets.QMenu(self.menuFile)
        self.menuLoad_Cartography_Labview.setObjectName("menuLoad_Cartography_Labview")
        self.menuAdd_Cartography = QtWidgets.QMenu(self.menuFile)
        self.menuAdd_Cartography.setObjectName("menuAdd_Cartography")
        self.menuExport = QtWidgets.QMenu(self.menuFile)
        self.menuExport.setObjectName("menuExport")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionLoad_TDMS = QtWidgets.QAction(MainWindow)
        self.actionLoad_TDMS.setObjectName("actionLoad_TDMS")
        self.actionLoad_Cartography_Labspec = QtWidgets.QAction(MainWindow)
        self.actionLoad_Cartography_Labspec.setObjectName("actionLoad_Cartography_Labspec")
        self.actionLoad_Cartography_separated_files = QtWidgets.QAction(MainWindow)
        self.actionLoad_Cartography_separated_files.setObjectName("actionLoad_Cartography_separated_files")
        self.actionLoad_1D_Trace = QtWidgets.QAction(MainWindow)
        self.actionLoad_1D_Trace.setObjectName("actionLoad_1D_Trace")
        self.actionLoad_Cartography_AFM = QtWidgets.QAction(MainWindow)
        self.actionLoad_Cartography_AFM.setObjectName("actionLoad_Cartography_AFM")
        self.actionDelete_Tree_Element = QtWidgets.QAction(MainWindow)
        self.actionDelete_Tree_Element.setObjectName("actionDelete_Tree_Element")
        self.actionLabview = QtWidgets.QAction(MainWindow)
        self.actionLabview.setObjectName("actionLabview")
        self.actionSeparated_Files = QtWidgets.QAction(MainWindow)
        self.actionSeparated_Files.setObjectName("actionSeparated_Files")
        self.actionAFM = QtWidgets.QAction(MainWindow)
        self.actionAFM.setObjectName("actionAFM")
        self.actionLabview_add = QtWidgets.QAction(MainWindow)
        self.actionLabview_add.setObjectName("actionLabview_add")
        self.actionLabspec_add = QtWidgets.QAction(MainWindow)
        self.actionLabspec_add.setObjectName("actionLabspec_add")
        self.actionSeparated_Files_add = QtWidgets.QAction(MainWindow)
        self.actionSeparated_Files_add.setObjectName("actionSeparated_Files_add")
        self.actionAFM_add = QtWidgets.QAction(MainWindow)
        self.actionAFM_add.setObjectName("actionAFM_add")
        self.action2D_image_as_txt = QtWidgets.QAction(MainWindow)
        self.action2D_image_as_txt.setObjectName("action2D_image_as_txt")
        self.actionQuit = QtWidgets.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionLabspec_dual_mode = QtWidgets.QAction(MainWindow)
        self.actionLabspec_dual_mode.setObjectName("actionLabspec_dual_mode")
        self.actionLabspec_Dual_Mode = QtWidgets.QAction(MainWindow)
        self.actionLabspec_Dual_Mode.setObjectName("actionLabspec_Dual_Mode")
        self.actionLabspec = QtWidgets.QAction(MainWindow)
        self.actionLabspec.setObjectName("actionLabspec")
        self.actionLabspec_Dual_Mode_add = QtWidgets.QAction(MainWindow)
        self.actionLabspec_Dual_Mode_add.setObjectName("actionLabspec_Dual_Mode_add")
        self.menuLoad_Cartography_Labview.addAction(self.actionLabview)
        self.menuLoad_Cartography_Labview.addAction(self.actionLabspec)
        self.menuLoad_Cartography_Labview.addAction(self.actionLabspec_Dual_Mode)
        self.menuLoad_Cartography_Labview.addAction(self.actionSeparated_Files)
        self.menuLoad_Cartography_Labview.addAction(self.actionAFM)
        self.menuLoad_Cartography_Labview.addSeparator()
        self.menuAdd_Cartography.addAction(self.actionLabview_add)
        self.menuAdd_Cartography.addAction(self.actionLabspec_add)
        self.menuAdd_Cartography.addAction(self.actionLabspec_Dual_Mode_add)
        self.menuAdd_Cartography.addAction(self.actionSeparated_Files_add)
        self.menuAdd_Cartography.addAction(self.actionAFM_add)
        self.menuExport.addAction(self.action2D_image_as_txt)
        self.menuFile.addAction(self.actionLoad_TDMS)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.menuLoad_Cartography_Labview.menuAction())
        self.menuFile.addAction(self.menuAdd_Cartography.menuAction())
        self.menuFile.addAction(self.actionLoad_1D_Trace)
        self.menuFile.addAction(self.actionDelete_Tree_Element)
        self.menuFile.addAction(self.menuExport.menuAction())
        self.menuFile.addAction(self.actionQuit)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget_2.setCurrentIndex(1)
        self.tabWidget.setCurrentIndex(0)
        self.toolBox_Analysis.setCurrentIndex(1)
        self.toolBox.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_2), _translate("MainWindow", "Plotting"))
        self.Open_Tree.setText(_translate("MainWindow", "Open Tree"))
        self.Open_Tree_Selected.setText(_translate("MainWindow", "Open Selected"))
        self.Close_Tree.setText(_translate("MainWindow", "Close Tree"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_3), _translate("MainWindow", "Data Management"))
        self.save_process_pb.setText(_translate("MainWindow", "Save Process"))
        self.process_data_pb.setText(_translate("MainWindow", "Process Data"))
        self.result_blue_label.setText(_translate("MainWindow", "Result (x):"))
        self.result_green_label.setText(_translate("MainWindow", "Result (x):"))
        self.result_red_label.setText(_translate("MainWindow", "Result (x):"))
        self.merge_spectra_cb.setText(_translate("MainWindow", "Merge spectra around ROI"))
        self.label_8.setText(_translate("MainWindow", "Enter comments concerning analysis:"))
        self.comments_edit.setToolTip(_translate("MainWindow", "Enter comments to be saved with analysis"))
        self.label_7.setText(_translate("MainWindow", "Formula\n"
"(the variable is \"x\"):"))
        self.label_20.setText(_translate("MainWindow", "<html><head/><body><p align=\"center\">Operation: (ex: a*x+b)</p><p align=\"center\">(ex: x if x&gt;0 else max(x))</p></body></html>"))
        self.measure_subtype_combo.setItemText(0, _translate("MainWindow", "dx"))
        self.measure_subtype_combo.setItemText(1, _translate("MainWindow", "amplitude"))
        self.measure_subtype_combo.setItemText(2, _translate("MainWindow", "x0"))
        self.label_9.setText(_translate("MainWindow", "Measurement Type: "))
        self.label_11.setText(_translate("MainWindow", "Measurement SubType: "))
        self.measure_type_combo.setItemText(0, _translate("MainWindow", "Cursor Intensity Integration"))
        self.measure_type_combo.setItemText(1, _translate("MainWindow", "Max"))
        self.measure_type_combo.setItemText(2, _translate("MainWindow", "Min"))
        self.measure_type_combo.setItemText(3, _translate("MainWindow", "Gaussian Fit"))
        self.measure_type_combo.setItemText(4, _translate("MainWindow", "Lorentzian Fit"))
        self.measure_type_combo.setItemText(5, _translate("MainWindow", "Exponential Decay Fit"))
        self.label_2.setText(_translate("MainWindow", "Formula parameters:"))
        self.do_operation_cb.setText(_translate("MainWindow", "Do operation"))
        self.toolBox_Analysis.setItemText(self.toolBox_Analysis.indexOf(self.page_measurement), _translate("MainWindow", "Measurement parameters"))
        self.apply_bkg_check.setText(_translate("MainWindow", "Apply Bkg"))
        self.set_whole_as_bkg_cb.setText(_translate("MainWindow", "Set whole carto as Bkg"))
        self.bkg_type.setItemText(0, _translate("MainWindow", "Sub."))
        self.bkg_type.setItemText(1, _translate("MainWindow", "Ratio"))
        self.do_zero_cb.setText(_translate("MainWindow", "Zero for negative values"))
        self.set_as_bkg_pb.setText(_translate("MainWindow", "Set as Bkg"))
        self.toolBox_Analysis.setItemText(self.toolBox_Analysis.indexOf(self.page_bkg), _translate("MainWindow", "Background settings"))
        self.show_despiked_cb.setText(_translate("MainWindow", "Show curve"))
        self.label_12.setText(_translate("MainWindow", "Threshold:"))
        self.remove_spikes_cb.setText(_translate("MainWindow", "Remove spikes"))
        self.label_16.setText(_translate("MainWindow", "Spikes width (pxls):"))
        self.label_17.setText(_translate("MainWindow", "Average width (pxls):"))
        self.label_18.setText(_translate("MainWindow", "Nloop:"))
        self.toolBox_Analysis.setItemText(self.toolBox_Analysis.indexOf(self.page_spikes), _translate("MainWindow", "Remove Spikes"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("MainWindow", "Analysis"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_TreeProperties), _translate("MainWindow", "Tree Properties"))
        self.label_15.setText(_translate("MainWindow", "Title"))
        self.label_14.setText(_translate("MainWindow", "Ylabel"))
        self.label_10.setText(_translate("MainWindow", "Aspect:"))
        self.aspect_combo.setItemText(0, _translate("MainWindow", "auto"))
        self.aspect_combo.setItemText(1, _translate("MainWindow", "equal"))
        self.plot_2D_pb.setText(_translate("MainWindow", "Plot 2D graph"))
        self.plot_1D_pb.setText(_translate("MainWindow", "Plot 1D graph"))
        self.label.setText(_translate("MainWindow", "Cmap:"))
        self.colorbar_cb.setText(_translate("MainWindow", "Colorbar"))
        self.update_plot_pb.setText(_translate("MainWindow", "Update"))
        self.save_plotting_pb.setText(_translate("MainWindow", "Save as"))
        self.label_13.setText(_translate("MainWindow", "Xlabel:"))
        self.use_labels_cb.setText(_translate("MainWindow", "Use Labels"))
        self.xlabel_edit.setText(_translate("MainWindow", "Xaxis (µm)"))
        self.ylabel_edit.setText(_translate("MainWindow", "Yaxis (µm)"))
        self.title_edit.setText(_translate("MainWindow", "ex: Photoluminescence map"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_plotting), _translate("MainWindow", "Plotting"))
        self.label_5.setText(_translate("MainWindow", "Current Group:"))
        self.label_4.setText(_translate("MainWindow", "Current Carto:"))
        self.label_6.setText(_translate("MainWindow", "Current Channel:"))
        self.label_3.setText(_translate("MainWindow", "Loaded Path:"))
        self.Filenamepath.setPlainText(_translate("MainWindow", "D:\\Data\\2016\\20161129"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page), _translate("MainWindow", "Data infos"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuLoad_Cartography_Labview.setTitle(_translate("MainWindow", "Load Cartography"))
        self.menuAdd_Cartography.setTitle(_translate("MainWindow", "Add Cartography"))
        self.menuExport.setTitle(_translate("MainWindow", "Export"))
        self.actionLoad_TDMS.setText(_translate("MainWindow", "Load TDMS"))
        self.actionLoad_Cartography_Labspec.setText(_translate("MainWindow", "Load Cartography Labspec"))
        self.actionLoad_Cartography_separated_files.setText(_translate("MainWindow", "Load Cartography separated files "))
        self.actionLoad_1D_Trace.setText(_translate("MainWindow", "Load 1D Trace"))
        self.actionLoad_Cartography_AFM.setText(_translate("MainWindow", "Load Cartography AFM"))
        self.actionDelete_Tree_Element.setText(_translate("MainWindow", "Delete Tree Element"))
        self.actionLabview.setText(_translate("MainWindow", "Labview"))
        self.actionSeparated_Files.setText(_translate("MainWindow", "Separated Files"))
        self.actionAFM.setText(_translate("MainWindow", "AFM"))
        self.actionLabview_add.setText(_translate("MainWindow", "Labview"))
        self.actionLabspec_add.setText(_translate("MainWindow", "Labspec"))
        self.actionSeparated_Files_add.setText(_translate("MainWindow", "Separated Files"))
        self.actionAFM_add.setText(_translate("MainWindow", "AFM"))
        self.action2D_image_as_txt.setText(_translate("MainWindow", "2D image as *.txt"))
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
        self.actionLabspec_dual_mode.setText(_translate("MainWindow", "Labspec dual mode"))
        self.actionLabspec_Dual_Mode.setText(_translate("MainWindow", "Labspec Dual Mode"))
        self.actionLabspec.setText(_translate("MainWindow", "Labspec"))
        self.actionLabspec_Dual_Mode_add.setText(_translate("MainWindow", "Labspec Dual Mode"))

from pyqtgraph import PlotWidget
import QtDesigner_ressources_rc

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())




# -*- coding: utf-8 -*-
"""
Created on Fri Aug 30 12:21:56 2019

@author: Weber
"""
from PyQt5.QtCore import QObject, pyqtSignal, pyqtSlot, QThread
from PyQt5 import QtWidgets
import socket
import select
import numpy as np
from bitstring import BitArray
from pymodaq.daq_utils.daq_utils import check_received_length, ThreadCommand, \
    getLineInfo, send_scalar, send_string, send_list, get_scalar, get_int, get_string, send_array
from pyqtgraph.parametertree import Parameter, ParameterTree
import pyqtgraph.parametertree.parameterTypes as pTypes
import pymodaq.daq_utils.custom_parameter_tree as custom_tree
from collections import OrderedDict

tcp_parameters = [{'title': 'Port:', 'name': 'port_id', 'type': 'int', 'value': 6341, 'default': 6341},
                                 {'title': 'IP:', 'name': 'socket_ip', 'type': 'str', 'value': 'localhost',
                                  'default': '10.47.1.33'},
                                 {'title': 'Infos Client:', 'name': 'infos', 'type': 'group', 'children': []},
                                 {'title': 'Connected clients:', 'name': 'conn_clients', 'type': 'table',
                                  'value': dict(), 'header': ['Type', 'adress']},]
# %%

class TCPClient(QObject):
    cmd_signal = pyqtSignal(ThreadCommand)
    params = []

    def __init__(self, parent=None, ipaddress="192.168.1.62", port=6341, params_state=None, client_type="GRABBER"):
        super().__init__()

        self.ipaddress = ipaddress
        self.port = port

        self.settings = Parameter.create(name='Settings', type='group', children=self.params)
        if params_state is not None:
            if isinstance(params_state, dict):
                self.settings.restoreState(params_state)
            elif isinstance(params_state, Parameter):
                self.settings.restoreState(params_state.saveState())

        self.parent = parent
        self.socket = None
        self.client_type = client_type #"GRABBER" or "ACTUATOR"

    def send_data(self, data_list):
        # first send 'Done' and then send the length of the list
        send_string(self.socket, 'Done')
        send_list(self.socket, data_list)



    def send_infos_xml(self, infos):
        send_string(self.socket, 'Infos')
        send_string(self.socket, infos)


    @pyqtSlot(ThreadCommand)
    def queue_command(self, command=ThreadCommand()):
        """

        """
        if command.command == "ini_connection":
            status = self.init_connection()

        elif command.command == "quit":
            try:
                self.socket.close()
            except Exception as e:
                pass
            finally:
                self.cmd_signal.emit(ThreadCommand('disconnected'))

        elif command.command == 'update_connection':
            self.ipaddress = command.attributes['ipaddress']
            self.port = command.attributes['port']

        elif command.command == 'data_ready':
            self.data_ready(command.attributes)

        elif command.command == 'send_info':
            path = command.attributes['path']
            param = command.attributes['param']

            param_xml = custom_tree.parameter_to_xml_string(param)

            send_string(self.socket, 'Info_xml')
            send_list(self.socket, path)

            # send value
            data = custom_tree.parameter_to_xml_string(param)
            send_string(self.socket, data)

        elif command.command == 'position_is':
            send_string(self.socket, 'position_is')
            send_scalar(self.socket, command.attributes[0])

        elif command.command == 'move_done':
            send_string(self.socket, 'move_done')
            send_scalar(self.socket, command.attributes[0])

        elif command.command == 'x_axis':
            send_string(self.socket, 'x_axis')
            x_axis = dict(label='', units='')
            if isinstance(command.attributes[0], np.ndarray):
                x_axis['data'] = command.attributes[0]
            elif isinstance(command.attributes[0], dict):
                x_axis.update(command.attributes[0].copy())

            send_array(self.socket, x_axis['data'])
            send_string(self.socket, x_axis['label'])
            send_string(self.socket, x_axis['units'])

        elif command.command == 'y_axis':
            send_string(self.socket, 'y_axis')
            y_axis = dict(label='', units='')
            if isinstance(command.attributes[0], np.ndarray):
                y_axis['data'] = command.attributes[0]
            elif isinstance(command.attributes[0], dict):
                y_axis.update(command.attributes[0].copy())

            send_array(self.socket, y_axis['data'])
            send_string(self.socket, y_axis['label'])
            send_string(self.socket, y_axis['units'])


    def init_connection(self):
        # %%
        try:
            # create an INET, STREAMing socket
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # now connect to the web server on port 80 - the normal http port
            self.socket.connect((self.ipaddress, self.port))
            self.cmd_signal.emit(ThreadCommand('connected'))
            send_string(self.socket, self.client_type)

            self.send_infos_xml(custom_tree.parameter_to_xml_string(self.settings))
            self.cmd_signal.emit(ThreadCommand('get_axis'))
            # %%
            while True:

                try:
                    ready_to_read, ready_to_write, in_error = \
                        select.select([self.socket], [self.socket], [self.socket], 0)

                    if len(ready_to_read) != 0:
                        message = get_string(self.socket)
                        # print(message)
                        self.get_data(message)

                    if len(in_error) != 0:
                        self.cmd_signal.emit(ThreadCommand('disconnected'))

                    QtWidgets.QApplication.processEvents()

                except Exception as e:
                    try:
                        self.cmd_signal.emit(ThreadCommand('Update_Status', [getLineInfo() + str(e), 'log']))
                        send_string(self.socket, 'Quit')
                        self.socket.close()
                    except:
                        pass
                    finally:
                        break

        except ConnectionRefusedError as e:
            self.cmd_signal.emit(ThreadCommand('disconnected'))
            self.cmd_signal.emit(ThreadCommand('Update_Status', [getLineInfo() + str(e), 'log']))

    # %%
    def get_data(self, message):
        """

        Parameters
        ----------
        message

        Returns
        -------

        """
        messg = ThreadCommand(message)

        if message == 'set_info':
            list_len = get_int(self.socket)
            path = []
            for ind in range(list_len):
                path.append(get_string(self.socket))
            param_xml = get_string(self.socket)
            messg.attributes = [path, param_xml]

        elif message == 'move_abs':
            position = get_scalar(self.socket)
            messg.attributes = [position]

        elif message == 'move_rel':
            position = get_scalar(self.socket)
            messg.attributes = [position]

        self.cmd_signal.emit(messg)

        # data = 100 * np.random.rand(100, 200)
        # self.data_ready([data.astype(np.int)])






    @pyqtSlot(list)
    def data_ready(self, datas):
        self.send_data(datas[0]['data'])  # datas from viewer 0 and get 'data' key (within the ordereddict list of datas


class TCPServer(QObject):
    """
    Abstract class to be used as inherited by DAQ_Viewer_TCP or DAQ_Move_TCP
    """
    def __init__(self, client_type='GRABBER'):
        QObject.__init__(self)

        self.connected_clients = []
        self.listening = True
        self.processing = False
        self.client_type = client_type

    def close_server(self):
        """
            close the current opened server.
            Update the settings tree consequently.

            See Also
            --------
            set_connected_clients_table, daq_utils.ThreadCommand
        """
        for sock_dict in self.connected_clients:
            try:
                sock_dict['socket'].close()
            except:
                pass
        self.connected_clients = []
        self.settings.child(('conn_clients')).setValue(self.set_connected_clients_table())

    def init_server(self):
        self.emit_status(ThreadCommand("Update_Status", [
            "Started new server for {:s}:{:d}".format(self.settings.child(('socket_ip')).value(),
                                                      self.settings.child(('port_id')).value()), 'log']))
        self.serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.serversocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        # bind the socket to a public host, and a well-known port
        try:
            # self.serversocket.bind((self.settings.child(('socket_ip')).value(),self.settings.child(('port_id')).value()))
            self.serversocket.bind((socket.gethostname(), self.settings.child(('port_id')).value()))
        except socket.error as msg:
            self.emit_status(ThreadCommand("Update_Status",
                                           ['Bind failed. Error Code : ' + str(msg.errno) + ' Message ' + msg.strerror,
                                            'log']))
            raise Exception('Bind failed. Error Code : ' + str(msg.errno) + ' Message ' + msg.strerror)

        self.serversocket.listen(1)
        self.connected_clients.append(dict(socket=self.serversocket, type='server'))
        self.settings.child(('conn_clients')).setValue(self.set_connected_clients_table())

        self.timer = self.startTimer(100)  # Timer event fired every 100ms
        # self.listen_client()

    def timerEvent(self, event):
        """
            Called by set timers.
            If the process is free, start the listen_client function.

            =============== ==================== ==============================================
            **Parameters**   **Type**              **Description**

            *event*          QTimerEvent object    Containing id from timer issuing this event
            =============== ==================== ==============================================

            See Also
            --------
            listen_client
        """
        if not self.processing:
            self.listen_client()

    def set_connected_clients_table(self):
        """

        """
        con_clients = OrderedDict()
        for socket_dict in self.connected_clients:
            try:
                address = str(socket_dict['socket'].getsockname())
            except:
                address = ""
            con_clients[socket_dict['type']] = address
        return con_clients

    @pyqtSlot(list)
    def print_status(self, status):
        """
            Print the given status.

            =============== ============= ================================================
            **Parameters**    **Type**       **Description**
            *status*          string list    a string list representing the status socket
            =============== ============= ================================================
        """
        print(status)

    def listen_client(self):
        """
            Server function.
            Used to listen incoming message from a client.
            Start a connection and :
            * if current socket corresponding to the serversocket attribute :

                * Read received command
                * Send the 'Update_Status' thread command if needed (log is not valid)

            * Else, in case of :

                * data received from client : process it reading commands from sock. Process the command or quit if asked.
                * client disconnected : remove from socket list


            See Also
            --------
            find_socket_type_within_connected_clients, set_connected_clients_table, daq_utils.ThreadCommand, read_commands, process_cmds, utility_classes.DAQ_Viewer_base.emit_status
        """
        try:
            self.processing = True
            # QtWidgets.QApplication.processEvents() #to let external commands in
            read_sockets, write_sockets, error_sockets = select.select(
                [client['socket'] for client in self.connected_clients], [],
                [client['socket'] for client in self.connected_clients],
                0)
            for sock in error_sockets:
                sock_type = self.find_socket_type_within_connected_clients(sock)
                if sock_type is not None:
                    self.connected_clients.remove(dict(socket=sock, type=sock_type))
                    self.settings.child(('conn_clients')).setValue(self.set_connected_clients_table())
                    try:
                        sock.close()
                    except:
                        pass
                    self.emit_status(ThreadCommand("Update_Status", ['Client ' + sock_type + ' disconnected', 'log']))

            for sock in read_sockets:
                QThread.msleep(100)
                # New connection
                if sock == self.serversocket:
                    (client_socket, address) = self.serversocket.accept()
                    # client_socket.setblocking(False)
                    DAQ_type = self.read_commands(client_socket)
                    if DAQ_type not in self.socket_types:
                        self.emit_status(ThreadCommand("Update_Status", [DAQ_type + ' is not a valid type', 'log']))
                        break

                    self.connected_clients.append(dict(socket=client_socket, type=DAQ_type))
                    self.settings.child(('conn_clients')).setValue(self.set_connected_clients_table())
                    self.emit_status(ThreadCommand("Update_Status",
                                                   [DAQ_type + ' connected with ' + address[0] + ':' + str(address[1]),
                                                    'log']))
                    QtWidgets.QApplication.processEvents()
                # Some incoming message from a client
                else:
                    # Data received from client, process it
                    try:
                        message = self.read_commands(sock)
                        if message == 'Done' or message == 'Info' or message == 'Infos' or message == 'Info_xml' or message == 'position_is' or message == 'move_done':
                            self.process_cmds(message)
                        elif message == 'Quit':
                            raise Exception("socket disconnect by user")
                        else:
                            self.process_cmds(message, command_sock=sock)

                    # client disconnected, so remove from socket list
                    except Exception as e:
                        sock_type = self.find_socket_type_within_connected_clients(sock)
                        if sock_type is not None:
                            self.connected_clients.remove(dict(socket=sock, type=sock_type))
                            self.settings.child(('conn_clients')).setValue(self.set_connected_clients_table())
                            # sock.shutdown(socket.SHUT_RDWR)
                            sock.close()
                            self.emit_status(
                                ThreadCommand("Update_Status", ['Client ' + sock_type + ' disconnected', 'log']))

            self.processing = False

        except Exception as e:
            self.emit_status(ThreadCommand("Update_Status", [str(e), 'log']))

    def read_commands(self, sock):
        """
            Read the commands from the given socket.

            =============== ============
            **Parameters**    **Type**
            *sock*
            =============== ============

            Returns
            -------
            message_bytes
                The readed and decoded message

            See Also
            --------
            check_received_length
        """
        message = get_string(sock)
        return message

    def send_command(self, sock, command="move_at"):
        """
            Send one of the message contained in self.message_list toward a socket with identity socket_type.
            First send the length of the command with 4bytes.

            =============== =========== ==========================
            **Parameters**    **Type**    **Description**
            *sock*             ???        The current socket
            *command*         string      The command as a string
            =============== =========== ==========================

            See Also
            --------
            utility_classes.DAQ_Viewer_base.emit_status, daq_utils.ThreadCommand, message_to_bytes
        """
        if command not in self.message_list:
            self.emit_status(
                ThreadCommand("Update_Status", ['Command: ' + str(command) + ' not in the specified list', 'log']))
            return

        if sock is not None:
            send_string(sock, command)

    def find_socket_within_connected_clients(self, client_type):
        """
            Find a socket from a conneceted client with socket type corresponding.

            =============== =========== ================================
            **Parameters**    **Type**    **Description**
            *client_type*      string     The corresponding client type
            =============== =========== ================================

            Returns
            -------
            dictionnary
                the socket dictionnary
        """
        res = None
        for socket_dict in self.connected_clients:
            if socket_dict['type'] == client_type:
                res = socket_dict['socket']
        return res

    def find_socket_type_within_connected_clients(self, sock):
        """
            Find a socket type from a connected client with socket content corresponding.

            =============== =========== ===================================
            **Parameters**    **Type**   **Description**
            *sock*             ???       The socket content corresponding.
            =============== =========== ===================================

            Returns
            -------
            dictionnary
                the socket dictionnary
        """
        res = None
        for socket_dict in self.connected_clients:
            if socket_dict['socket'] == sock:
                res = socket_dict['type']
        return res

    def read_data(self, sock):
        pass

    def send_data(self, sock, data):
        pass

    def command_done(self, command_sock):
        pass

    def command_to_from_client(self, command):
        pass

    def process_cmds(self, command, command_sock=None):
        """
            Process the given command.

            Depending on the command name :
            * Done :

                * Find a socket from the 'GRABBER' connected client.
                * Send data to a viewer or a client (depending on command_sock).

            * Info : Find a socket from the 'GRABBER' connected client and read infos.

            * Send Data 0D :

                * Find a socket from the 'GRABBER' connected client.
                * set a 1D Mock data.

            * Send Data 1D :

                * Find a socket from the 'GRABBER' connected client.
                * set a 1D Mock data.

            * Send Data 2D :

                * Find a socket from the 'GRABBER' connected client.
                * set a 2D Mock data.

            =============== =========== ==========================
            **Parameters**    **Type**    **Description**
            *command*         string      the command as a string
            *command_sock*    ???
            =============== =========== ==========================

            See Also
            --------
            find_socket_within_connected_clients, read_data, send_data, utility_classes.DAQ_Viewer_base.emit_status, daq_utils.ThreadCommand, send_command, set_1D_Mock_data, set_2D_Mock_data, process_cmds
        """
        if command not in self.message_list:
            return

        if command == 'Done':  # means the given socket finished grabbing data and is ready to send them
            self.command_done(command_sock)


        elif command == "Infos":
            try:
                sock = self.find_socket_within_connected_clients(self.client_type)
                if sock is not None:  # if client self.client_type is connected then send it the command
                    self.read_infos(sock)


            except Exception as e:
                self.emit_status(ThreadCommand("Update_Status", [str(e), 'log']))

        elif command == "Info":
            try:
                sock = self.find_socket_within_connected_clients(self.client_type)
                if sock is not None:  # if client self.client_type is connected then send it the command
                    self.read_info(sock)
            except Exception as e:
                self.emit_status(ThreadCommand("Update_Status", [str(e), 'log']))

        elif command == 'Info_xml':
            sock = self.find_socket_within_connected_clients(self.client_type)
            if sock is not None:
                list_len = get_int(sock)
                path = []
                for ind in range(list_len):
                    data_len = int.from_bytes(check_received_length(sock, 4), 'big')
                    path.append(check_received_length(sock, data_len).decode())
                data_len = int.from_bytes(check_received_length(sock, 4), 'big')
                param_xml = check_received_length(sock, data_len).decode()
                param_dict = custom_tree.XML_string_to_parameter(param_xml)[0]

                param_here = self.settings.child('infos', *path[1:])
                param_here.restoreState(param_dict)

        else:
            self.command_to_from_client(command)



    def read_infos(self, sock):
        length_bytes = check_received_length(sock, 4)
        length = np.frombuffer(length_bytes, dtype='>i4')[0]  # big endian 4 bytes==uint32 swaped
        infos = check_received_length(sock, length).decode()
        params = custom_tree.XML_string_to_parameter(infos)
        param_state = {'title': 'Infos Client:', 'name': 'infos', 'type': 'group', 'children': params}
        self.settings.child(('infos')).restoreState(param_state)

    def read_info(self, sock, dtype=np.float64):
        """
        """
        try:

            ##first get the info type
            length_bytes = check_received_length(sock, 4)
            length = np.frombuffer(length_bytes, dtype='>i4')[0]  # big endian 4 bytes==uint32 swaped
            message = check_received_length(sock, length).decode()

            # get data length
            length_bytes = check_received_length(sock, 4)
            length = np.frombuffer(length_bytes, dtype='>i4')[0]

            ##then get data
            data_bytes = check_received_length(sock, length)

            data = np.frombuffer(data_bytes, dtype=dtype)
            data = data.newbyteorder()[0]  # convert it to big endian
            try:
                self.settings.child('infos', message).setValue(data)
            except Exception as e:
                self.emit_status(ThreadCommand('Update_Status', [str(e), 'log']))

        except Exception as e:
            data = 0

        return data

if __name__ ==  '__main__':
    tcp_client = TCPClient()
    tcp_client.init_connection()


from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import QObject, pyqtSlot, QThread, pyqtSignal, QRectF, QRect, QPointF
import sys
import numpy as np
from collections import OrderedDict
from pyqtgraph import gaussianFilter, ROI, RectROI, PolyLineROI, Point


import pyqtgraph.parametertree.parameterTypes as pTypes
from pyqtgraph.parametertree import Parameter, ParameterTree
import pymodaq.daq_utils.custom_parameter_tree
from pyqtgraph.dockarea import DockArea, Dock
from easydict import EasyDict as edict

class PolyLineROI_custom(PolyLineROI):
    def __init__(self,*args,**kwargs):
        super(PolyLineROI_custom,self).__init__(*args,**kwargs)

    def get_vertex(self):
        return [h['item'].pos() for h in self.handles]

    def getArrayIndexes(self, spacing=1, **kwds):
        imgPts = self.get_vertex()
        positions=[]
        for i in range(len(imgPts) - 1):
            d = Point(imgPts[i + 1] - imgPts[i])
            o = Point(imgPts[i])
            vect=Point(d.norm())
            Npts=0
            while Npts*spacing < d.length():

                positions.append(((o+Npts*spacing*vect).x(),(o+Npts*spacing*vect).y()))
                Npts+=1

        return positions


class ScanSelector(QObject):

    scan_select_signal = pyqtSignal(ROI)

    params = [
        {'title': 'Scan options', 'name': 'scan_options', 'type': 'group', 'children': [
            {'title': 'Sources:', 'name': 'sources', 'type': 'list',},
            {'title': 'Viewers:', 'name': 'viewers', 'type': 'list', },
            {'title': 'Scan type:', 'name': 'scan_type', 'type': 'list', 'values': ['Scan1D', 'Scan2D'], 'value': 'Scan2D'},
            ]},
        {'title': 'Scan Area', 'name': 'scan_area', 'type': 'group', 'children': [
            {'title': 'ROI select:', 'name': 'ROIselect', 'type': 'group', 'visible': True, 'children': [
                {'title': 'x0:', 'name': 'x0', 'type': 'int', 'value': 0, 'min': 0},
                {'title': 'y0:', 'name': 'y0', 'type': 'int', 'value': 0, 'min': 0},
                {'title': 'width:', 'name': 'width', 'type': 'int', 'value': 10, 'min': 1},
                {'title': 'height:', 'name': 'height', 'type': 'int', 'value': 10, 'min': 1},
            ]},
            {'title': 'Coordinates:', 'name': 'coordinates', 'type': 'itemselect', 'visible': True},
            ]},
        ]

    def __init__(self, viewer_items =[], scan_type='Scan2D', positions=[]):
        """

        Parameters
        ----------
        viewer_items: dict where the keys are the titles of the sources while the values are dict with keys
                        viewers: list of plotitems
                        names: list of viewer titles
        scan_type: (str) either 'Scan1D' correspondign to a polyline ROI or 'Scan2D' for a rect Roi
        positions: list
                        in case of 'Scan1D', should be a sequence of 2 floats sequence [(x1,y1),(x2,y2),(x3,y3),...]
                        in case of 'Scan2D', should be a sequence of 4 floats (x, y , w, h)
        """
        super(ScanSelector, self).__init__()
        self._viewers_items = viewer_items
        self.sources_names = list(viewer_items.keys())
        if len(viewer_items) != 0:
            self.scan_selector_source = viewer_items[self.sources_names[0]]['viewers'][0]
        else:
            self.scan_selector_source = None
        self.scan_selector = None
        self.setupUI()
        self.settings.child('scan_options', 'scan_type').setValue(scan_type)

        self.remove_scan_selector()
        if self.scan_selector_source is not None:
            if len(viewer_items[self.sources_names[0]]['viewers']) == 1:
                self.settings.child('scan_options', 'viewers').hide()
        else:
            self.settings.child('scan_options', 'viewers').hide()
        self.update_scan_area_type()

        if scan_type == "Scan1D" and positions != []:
            self.scan_selector.setPoints(positions)
        elif scan_type == 'Scan2D' and positions != []:
            self.scan_selector.setPos(positions[:2])
            self.scan_selector.setSize(positions[3:])

    @property
    def viewers_items(self):
        return self._viewers_items

    @viewers_items.setter
    def viewers_items(self, items):
        self._viewers_items = items
        self.sources_names = list(items.keys())
        self.scan_selector_source = items[self.sources_names[0]]['viewers'][0]
        self.settings.child('scan_options', 'sources').setOpts(limits=self.sources_names)
        viewer_names = self._viewers_items[self.sources_names[0]]['names']
        self.settings.child('scan_options', 'viewers').setOpts(limits=viewer_names)

    def show(self, visible=True):
        self.show_scan_selector(visible)
        if visible:
            self.widget.show()
        else:
            self.widget.hide()

    def hide(self):
        self.show(False)

    def setupUI(self):
        self.widget = QtWidgets.QWidget()
        layout = QtWidgets.QVBoxLayout()

        self.settings_tree = ParameterTree()
        layout.addWidget(self.settings_tree,10)
        self.settings_tree.setMinimumWidth(300)
        self.settings=Parameter.create(name='Settings', type='group', children=self.params)
        self.settings_tree.setParameters(self.settings, showTop=False)

        self.settings.child('scan_options', 'sources').setOpts(limits=self.sources_names)
        if len(self.viewers_items):
            viewer_names = self._viewers_items[self.sources_names[0]]['names']
        else:
            viewer_names = []
        self.settings.child('scan_options', 'viewers').setOpts(limits=viewer_names)

        self.settings.sigTreeStateChanged.connect(self.source_changed)
        self.widget.setLayout(layout)

        #self.widget.show()
        self.widget.setWindowTitle('Scan Selector')

    def source_changed(self,param,changes):
        for param, change, data in changes:
            path = self.settings.childPath(param)
            if path is not None:
                childName = '.'.join(path)
            else:
                childName = param.name()
            if change == 'childAdded':
                pass

            elif change == 'value':
                if param.name() == 'sources' and param.value() is not None:
                    viewer_names= self._viewers_items[param.value()]['names']
                    self.settings.child('scan_options', 'viewers').setOpts(limits=viewer_names)
                    if len(viewer_names) == 1:
                        self.settings.child('scan_options', 'viewers').hide()

                    self.remove_scan_selector()
                    self.scan_selector_source = self._viewers_items[param.value()]['viewers'][0]
                    self.update_scan_area_type()

                if param.name() == 'scan_type':


                    if param.value() == 'Scan1D':
                        self.settings.child('scan_area', 'ROIselect').hide()
                        self.settings.child('scan_area', 'coordinates').show()
                        self.remove_scan_selector()
                        self.update_scan_area_type()
                    else:
                        self.settings.child('scan_area', 'ROIselect').show()
                        self.settings.child('scan_area', 'coordinates').hide()
                        self.remove_scan_selector()
                        self.update_scan_area_type()
                    self.scan_selector.sigRegionChangeFinished.emit(self.scan_selector)


            elif change == 'parent':
                pass

    def remove_scan_selector(self):
        if self.scan_selector_source is not None:
            try:
                self.scan_selector_source.image_widget.plotitem.removeItem(self.scan_selector)
            except:
                pass

    pyqtSlot(str)
    def update_scan_area_type(self):

        if self.settings.child('scan_options', 'scan_type').value() == 'Scan1D':
            scan_area_type = 'PolyLines'
        else:
            scan_area_type = 'Rect'


        self.remove_scan_selector()
        if scan_area_type == 'Rect':
            self.scan_selector = RectROI([0,0],[10,10])

        elif scan_area_type == 'PolyLines':
            self.scan_selector = PolyLineROI_custom([(0, 0), [10, 10]])
        if self.scan_selector_source is not None:
            self.scan_selector.sigRegionChangeFinished.connect(self.update_scan)
            self.scan_selector_source.image_widget.plotitem.addItem(self.scan_selector)
            self.show_scan_selector()

            self.scan_selector.sigRegionChangeFinished.emit(self.scan_selector)


    def show_scan_selector(self,visible=True):
        self.scan_selector.setVisible(visible)

    def update_scan(self, roi):
        if self.scan_selector_source is not None:
            if isinstance(roi, RectROI):
                self.settings.child('scan_area', 'ROIselect', 'x0').setValue(roi.pos().x())
                self.settings.child('scan_area', 'ROIselect', 'y0').setValue(roi.pos().y())
                self.settings.child('scan_area', 'ROIselect', 'width').setValue(roi.size().x())
                self.settings.child('scan_area', 'ROIselect', 'height').setValue(roi.size().y())
            elif isinstance(roi, PolyLineROI_custom):
                self.settings.child('scan_area', 'coordinates').setValue(dict(all_items=['({:.03f} , {:.03f})'.format(pt.x(),
                            pt.y()) for pt in roi.get_vertex()],selected=[]))

            self.scan_select_signal.emit(roi)

if __name__ == '__main__':
    class UI():
        def __init__(self):
            pass


    from pymodaq.daq_utils.plotting.viewer2D.viewer2D_main import Viewer2D
    class FakeDaqScan():

        def __init__(self, area):
            self.area = area
            self.detector_modules = None
            self.ui = UI()
            self.dock = Dock('2D scan', size=(500, 300), closable=False)

            form = QtWidgets.QWidget()
            self.ui.scan2D_graph = Viewer2D(form)
            self.dock.addWidget(form)
            self.area.addDock(self.dock)

    from pymodaq.daq_utils.daq_enums import DAQ_type
    from pymodaq.daq_viewer.daq_viewer_main import DAQ_Viewer


    app = QtWidgets.QApplication(sys.argv)
    win = QtWidgets.QMainWindow()
    area = DockArea()

    win.setCentralWidget(area)
    win.resize(1000, 500)
    win.setWindowTitle('pymodaq main')
    fake = FakeDaqScan(area)

    prog = DAQ_Viewer(area, title="Testing", DAQ_type=DAQ_type['DAQ2D'].name, parent_scan=fake)
    prog.ui.IniDet_pb.click()
    QThread.msleep(1000)
    QtWidgets.QApplication.processEvents()
    prog2 = DAQ_Viewer(area, title="Testing2", DAQ_type=DAQ_type['DAQ2D'].name, parent_scan=fake)
    prog2.ui.IniDet_pb.click()
    QThread.msleep(1000)
    QtWidgets.QApplication.processEvents()


    fake.detector_modules=[prog, prog2]
    items=OrderedDict()
    items[prog.title]=dict(viewers=[view for view in prog.ui.viewers],
                           names=[view.title for view in prog.ui.viewers],
                           )
    items[prog2.title] = dict(viewers=[view for view in prog2.ui.viewers],
                             names=[view.title for view in prog2.ui.viewers])
    items["DaqScan"] = dict(viewers=[fake.ui.scan2D_graph],
                             names=["DaqScan"])

    selector = ScanSelector(items,scan_type='Scan1D', positions=[(10,-10),(4,4),(80,50)])

    win.show()
    sys.exit(app.exec_())


import sys
import tables
from collections import OrderedDict
import datetime
import os
import numpy as np

from PyQt5 import QtGui, QtWidgets
from PyQt5.QtCore import QObject, pyqtSlot, pyqtSignal, QLocale, QDateTime, QRectF, QDate, QThread

from pyqtgraph.dockarea import Dock
from pymodaq.daq_utils.daq_utils import DockArea
from pyqtgraph.parametertree import Parameter, ParameterTree
import pyqtgraph.parametertree.parameterTypes as pTypes
import pymodaq.daq_utils.custom_parameter_tree as custom_tree
from pymodaq.daq_utils.daq_utils import select_file


from pymodaq.daq_viewer.daq_viewer_main import DAQ_Viewer
from pymodaq.daq_move.daq_move_main import DAQ_Move
from pymodaq.daq_utils.plotting.viewer0D.viewer0D_main import Viewer0D
from pymodaq.daq_utils.plotting.viewer1D.viewer1D_main import Viewer1D
from pymodaq.daq_utils.plotting.viewer2D.viewer2D_main import Viewer2D
from pymodaq.daq_utils.h5browser import browse_data
from pymodaq.daq_utils import daq_utils as utils
from pymodaq.daq_utils.h5browser import H5Browser


class CustomApp(QtWidgets.QWidget, QObject):
    #custom signal that will be fired sometimes. Could be connected to an external object method or an internal method
    log_signal = pyqtSignal(str)

    #list of dicts enabling the settings tree on the user interface
    params = [{'title': 'Main settings:', 'name': 'main_settings', 'type': 'group', 'children': [
                {'title': 'File name:', 'name': 'target_filename', 'type': 'str', 'value': "", 'readonly': True},
                {'title': 'Date:', 'name': 'date', 'type': 'date', 'value': QDate.currentDate()},
                {'title': 'Do something:', 'name': 'do_something', 'type': 'bool', 'value': False},
                {'title': 'Something done:', 'name': 'something_done', 'type': 'led', 'value': False, 'readonly': True},
                {'title': 'Infos:', 'name': 'info', 'type': 'text', 'value': ""},
                ]},
              {'title': 'Other settings:', 'name': 'other_settings', 'type': 'group', 'children': [
                  {'title': 'List of stuffs:', 'name': 'list_stuff', 'type': 'list', 'value': 'first',
                   'values': ['first', 'second', 'third'], 'tip': 'choose a stuff from the list'},
                  {'title': 'List of integers:', 'name': 'list_int', 'type': 'list', 'value': 0,
                   'values': [0, 256, 512], 'tip': 'choose a stuff from this int list'},
                  {'title': 'one integer:', 'name': 'an_integer', 'type': 'int', 'value': 500,},
                  {'title': 'one float:', 'name': 'a_float', 'type': 'float', 'value': 2.7,},
                  ]},
              ]

    def __init__(self, parent):
        QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
        super(CustomApp, self).__init__()
        if not isinstance(parent, DockArea):
            raise Exception('no valid parent container, expected a DockArea')
        self.dockarea = parent
        self.mainwindow = parent.parent()

        #init the object parameters
        self.one_parameter = None
        self.another_parameter = None
        self.detector = None
        self.save_file_pathname = None
        self.raw_data = []


        #init the user interface
        self.set_GUI()

    def set_GUI(self):
        ###########################################
        ###########################################
        #init the docks containing the main widgets

        #############################################
        #this one for the custom application settings
        dock_settings = Dock('Settings', size=(350, 350))
        self.dockarea.addDock(dock_settings, 'left')
        # create main parameter tree
        self.settings_tree = ParameterTree()
        dock_settings.addWidget(self.settings_tree, 10)
        self.settings_tree.setMinimumWidth(300)
        #create a Parameter object containing the settings
        self.settings = Parameter.create(name='Settings', type='group', children=self.params)
        #load the tree with this parameter object
        self.settings_tree.setParameters(self.settings, showTop=False)
        #any change to the tree on the user interface will call the parameter_tree_changed method where all actions will be applied
        self.settings.sigTreeStateChanged.connect(
            self.parameter_tree_changed)

        ################################################################
        #create a logger dock where to store info senf from the programm
        self.dock_logger = Dock("Logger")
        self.logger_list = QtWidgets.QListWidget()
        self.logger_list.setMinimumWidth(300)
        self.dock_logger.addWidget(self.logger_list)
        self.dockarea.addDock(self.dock_logger, 'bottom', dock_settings)
        # dock_logger.setVisible(False)
        #connect together this custom signal with the add_log method
        self.log_signal[str].connect(self.add_log)

        #######################################################################################################################
        #create a dock containing a viewer object, could be 0D, 1D or 2D depending what kind of data one want to plot here a 0D
        dock_Viewer0D = Dock('Viewer dock', size=(350, 350))
        self.dockarea.addDock(dock_Viewer0D, 'right', self.dock_logger)
        target_widget = QtWidgets.QWidget()
        self.target_viewer = Viewer0D(target_widget)
        dock_Viewer0D.addWidget(target_widget)

        ###################################################################################
        #create 2 docks to display the DAQ_Viewer (one for its settings, one for its viewer)
        dock_detector_settings = Dock("Detector Settings", size=(350, 350))
        self.dockarea.addDock(dock_detector_settings, 'right', dock_settings)
        dock_detector = Dock("Detector Viewer", size=(350, 350))
        self.dockarea.addDock(dock_detector, 'right', dock_detector_settings)
        #init one daq_viewer object named detector
        self.detector = DAQ_Viewer(self.dockarea, dock_settings=dock_detector_settings,
                                                 dock_viewer=dock_detector, title="A detector", DAQ_type='DAQ0D')
        #set its type to 'Mock'
        control_type = 'Mock'
        self.detector.ui.Detector_type_combo.setCurrentText(control_type)
        #init the detector and wait 1000ms for the completion
        self.detector.ui.IniDet_pb.click()
        self.detector.settings.child('main_settings', 'wait_time').setValue(100)
        QtWidgets.QApplication.processEvents()
        QThread.msleep(1000)
        self.detector.grab_done_signal.connect(self.data_done)

        #############################
        #create a dock for a DAQ_Move
        dock_move = Dock("Move module", size=(350, 350))
        self.dockarea.addDock(dock_move, 'right', self.dock_logger)
        move_widget = QtWidgets.QWidget()
        self.move = DAQ_Move(move_widget)
        dock_move.addWidget(move_widget)
        self.move.ui.IniStage_pb.click()
        QtWidgets.QApplication.processEvents()
        QThread.msleep(1000)





        ############################################
        # creating a menubar
        self.menubar = self.mainwindow.menuBar()
        self.create_menu(self.menubar)

        #creating a toolbar
        self.toolbar = QtWidgets.QToolBar()
        self.create_toolbar()
        self.mainwindow.addToolBar(self.toolbar)

    @pyqtSlot(OrderedDict)
    def data_done(self,data):
        print(data)


    @pyqtSlot(QRectF)
    def update_weighted_settings(self, rect):
        self.settings.child('weighting_settings', 'x0').setValue(int(rect.x()))
        self.settings.child('weighting_settings', 'y0').setValue(int(rect.y()))
        self.settings.child('weighting_settings', 'width').setValue(max([1, int(rect.width())]))
        self.settings.child('weighting_settings', 'height').setValue(max([1, int(rect.height())]))

    def parameter_tree_changed(self, param, changes):
        for param, change, data in changes:
            path = self.settings.childPath(param)
            if path is not None:
                childName = '.'.join(path)
            else:
                childName = param.name()
            if change == 'childAdded':
                pass

            elif change == 'value':
                if param.name() == 'do_something':
                    if param.value():
                        self.log_signal.emit('Do something')
                        self.detector.grab_done_signal.connect(self.show_data)
                        self.settings.child('main_settings', 'something_done').setValue(True)
                    else:
                        self.log_signal.emit('Stop Doing something')
                        self.detector.grab_done_signal.disconnect()
                        self.settings.child('main_settings', 'something_done').setValue(False)


            elif change == 'parent':
                pass

    @pyqtSlot(OrderedDict)
    def show_data(self,data):
        """
        do stuff with data from the detector if its grab_done_signal has been connected
        Parameters
        ----------
        data: (OrderedDict) #OrderedDict(name=self.title,x_axis=None,y_axis=None,z_axis=None,data0D=None,data1D=None,data2D=None)
        """
        data0D = [[data['data0D'][key]] for key in data['data0D']]
        self.raw_data.append(data0D)
        self.target_viewer.show_data(data0D)

    def create_menu(self, menubar):
        """
        """
        menubar.clear()

        #%% create file menu
        file_menu=menubar.addMenu('File')
        load_action=file_menu.addAction('Load file')
        load_action.triggered.connect(self.load_file)
        save_action=file_menu.addAction('Save file')
        save_action.triggered.connect(self.save_data)

        file_menu.addSeparator()
        quit_action=file_menu.addAction('Quit')
        quit_action.triggered.connect(self.quit_function)

        settings_menu=menubar.addMenu('Settings')
        docked_menu=settings_menu.addMenu('Docked windows')
        action_load=docked_menu.addAction('Load Layout')
        action_save=docked_menu.addAction('Save Layout')


    def load_file(self):
        #init the data browser module
        widg = QtWidgets.QWidget()
        self.data_browser = H5Browser(widg)
        widg.show()

    def quit_function(self):
        #close all stuff that need to be
        self.detector.quit_fun()
        QtWidgets.QApplication.processEvents()
        self.mainwindow.close()

    def create_toolbar(self):
        iconquit = QtGui.QIcon()
        iconquit.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/close2.png"), QtGui.QIcon.Normal,
                           QtGui.QIcon.Off)
        self.quit_action = QtWidgets.QAction(iconquit, "Quit program", None)
        self.toolbar.addAction(self.quit_action)
        self.quit_action.triggered.connect(self.quit_function)


        icon_detector = QtGui.QIcon()
        icon_detector.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/camera.png"), QtGui.QIcon.Normal,
                                  QtGui.QIcon.Off)
        self.detector_action = QtWidgets.QAction(icon_detector, "Grab from camera", None)
        self.detector_action.setCheckable(True)
        self.toolbar.addAction(self.detector_action)
        self.detector_action.triggered.connect(lambda: self.run_detector())

        iconload = QtGui.QIcon()
        iconload.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.loadaction = QtWidgets.QAction(iconload, "Load target file (.h5, .png, .jpg) or data from camera", None)
        self.toolbar.addAction(self.loadaction)
        self.loadaction.triggered.connect(self.load_file)


        iconsave = QtGui.QIcon()
        iconsave.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/SaveAs.png"), QtGui.QIcon.Normal,
                           QtGui.QIcon.Off)
        self.saveaction = QtWidgets.QAction(iconsave, "Save current data", None)
        self.toolbar.addAction(self.saveaction)
        self.saveaction.triggered.connect(self.save_data)

    def run_detector(self):
        self.detector.ui.grab_pb.click()


    def save_data(self):
        try:
            fname = utils.select_file(start_path=self.save_file_pathname, save=True, ext='h5')

            if not (not (fname)):
                with tables.open_file(str(fname), mode='w', title='an example h5 file name') as h5file:
                    data_to_save = np.squeeze(np.array(self.raw_data))
                    #save metadata
                    h5file.root._v_attrs['settings'] = custom_tree.parameter_to_xml_string(self.settings)
                    for ind in range(data_to_save.shape[1]):
                        arr = h5file.create_array('/', 'data_{:d}'.format(ind), data_to_save[:,ind])
                        arr._v_attrs['shape'] = data_to_save.shape[0]
                        arr._v_attrs['type'] = 'data1D'

                    arr = h5file.create_array('/', 'data_2D', data_to_save)
                    arr._v_attrs['shape'] = data_to_save.shape
                    arr._v_attrs['type'] = 'data2D'



                    logger = "logging"
                    text_atom = tables.atom.ObjectAtom()
                    logger_array = h5file.create_vlarray('/', logger, atom=text_atom)
                    logger_array._v_attrs['type'] = 'list'
                    for ind_log in range(self.logger_list.count()):
                        txt = self.logger_list.item(ind_log).text()
                        logger_array.append(txt)

                st = 'file {:s} has been saved'.format(fname)
                self.add_log(st)
                self.settings.child('min_settings', 'info').setValue(st)


        except Exception as e:
            self.add_log(str(e))


    @pyqtSlot(str)
    def add_log(self, txt):
        """
            Add a log to the logger list from the given text log and the current time

            ================ ========= ======================
            **Parameters**   **Type**   **Description**

             *txt*             string    the log to be added
            ================ ========= ======================

        """
        now = datetime.datetime.now()
        new_item = QtWidgets.QListWidgetItem(str(now) + ": " + txt)
        self.logger_list.addItem(new_item)
        ##to do
        ##self.save_parameters.logger_array.append(str(now)+": "+txt)

    @pyqtSlot(str)
    def emit_log(self, txt):
        """
            Emit a log-signal from the given log index

            =============== ======== =======================
            **Parameters**  **Type** **Description**

             *txt*           string   the log to be emitted
            =============== ======== =======================

        """
        self.log_signal.emit(txt)


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    win = QtWidgets.QMainWindow()
    area = DockArea()
    win.setCentralWidget(area)
    win.resize(1000, 500)
    win.setWindowTitle('pymodaq example')
    prog = CustomApp(area)
    win.show()
    sys.exit(app.exec_())


from PyQt5 import QtGui, QtWidgets
from PyQt5.QtCore import Qt,QObject, pyqtSlot, QThread, pyqtSignal, QLocale, QDateTime, QSize, QByteArray

import sys
import pymodaq
from pymodaq.daq_utils.plotting.viewer0D.viewer0D_main import Viewer0D
from pymodaq.daq_utils.plotting.viewer1D.viewer1D_main import Viewer1D
from pymodaq.daq_utils.plotting.viewer2D.viewer2D_main import Viewer2D
from pymodaq.daq_utils.daq_utils import select_file


from collections import OrderedDict
import numpy as np

from pyqtgraph.parametertree import Parameter, ParameterTree
import pyqtgraph.parametertree.parameterTypes as pTypes
import pymodaq.daq_utils.custom_parameter_tree as custom_tree
from pymodaq.daq_utils.tree_layout.tree_layout_main import Tree_layout
import pymodaq.daq_utils.daq_utils as utils
import os
from easydict import EasyDict as edict
from pyqtgraph.dockarea import DockArea, Dock
import tables



class DAQ_Analysis(QtWidgets.QWidget,QObject):
    """

        ======================== =========================================
        **Attributes**            **Type**

        *dockarea*                instance of pyqtgraph.DockArea
        *mainwindow*              instance of pyqtgraph.DockArea
        *title*                   string
        *waitime*                 int
        *h5file*                  instance class File from tables module
        *loaded_data*             2D array
        *loaded_data_scan_type*   string
        *x_axis*                  float array
        *y_axis*                  float array
        *data_buffer*             list of data
        *ui*                      QObject
        ======================== =========================================

        Raises
        ------
        parent Exception
            If parent argument is None in constructor abort

        See Also
        --------
        set_GUI


        References
        ----------
        PyQt5, pyqtgraph, QtWidgets, QObject

    """
    command_DAQ_signal=pyqtSignal(list)
    log_signal=pyqtSignal(str)

    def __init__(self,parent=None,title=None):
        QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
        super(DAQ_Analysis,self).__init__()
        if parent is None:
            raise Exception('no valid parent container, expected dockarea')
            # parent=DockArea()
            # exit(0)

        self.dockarea=parent
        self.mainwindow=parent.parent()


        if title is None:
            title='DAQ_Analysis'
        self.title=title
        self.mainwindow.setWindowTitle(self.title)

        self.wait_time=2000

        self.h5file=None
        self.loaded_data=None
        self.loaded_data_scan_type=None
        self.x_axis=None
        self.y_axis=None

        self.data_buffer=[] #convenience list to store data to be displayed

        self.ui=QObject() #the user interface
        self.set_GUI()

    def set_GUI(self):
        """
            Create the graphic interface of the h5 file analyser, including:
                * *h5 file dock* : QtreeWidget (custom instance of Tree_layout) showing the contents of the h5 file
                * *status bar* : the top_down information bar
                * *tree_dock* : The QTree viewer
                * *1D viewer dock* : the preview window of 1D data
                * *2D viewer dock* : the preview window of 2D data
                * *Navigator viewer dock* : the global navigator graphic interface

            See Also
            --------
            show_h5_attributes, show_h5_data, daq_utils.custom_parameter_tree.Table_custom, update_viewer_data
        """
        #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # h5 file dock
        V_splitter=QtWidgets.QSplitter(Qt.Vertical)
        H_splitter=QtWidgets.QSplitter(Qt.Horizontal)

        Form = QtWidgets.QWidget()
        self.ui.h5file_tree = Tree_layout(Form,col_counts=2,labels=["Node",'Pixmap'])
        self.ui.h5file_tree.ui.Tree.setMinimumWidth(250)
        self.ui.h5file_tree.ui.Tree.itemClicked.connect(self.show_h5_attributes)
        self.ui.h5file_tree.ui.Tree.itemDoubleClicked.connect(self.show_h5_data)
        V_splitter.addWidget(Form)
        self.ui.attributes_table=custom_tree.Table_custom()
        V_splitter.addWidget(self.ui.attributes_table)
        H_splitter.addWidget(V_splitter)
        self.ui.settings_tree=ParameterTree()
        self.ui.settings_tree.setMinimumWidth(300)
        H_splitter.addWidget(self.ui.settings_tree)
        self.ui.menubar=self.mainwindow.menuBar()
        self.create_menu(self.ui.menubar)

        #%%create status bar
        self.ui.statusbar=QtWidgets.QStatusBar()
        self.ui.statusbar.setMaximumHeight(25)


        #%% create tree dock
        file_dock = Dock("File data", size=(1, 1), autoOrientation=False)     ## give this dock the minimum possible size
        file_dock.setOrientation('vertical')
        file_dock.addWidget(H_splitter)
        file_dock.addWidget(self.ui.statusbar)
        self.settings=Parameter.create(name='Param', type='group')
        self.ui.settings_tree.setParameters(self.settings, showTop=False)

        ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        ##% 1D viewer Dock
        viewer1D_widget=QtWidgets.QWidget()
        self.ui.viewer1D=Viewer1D(viewer1D_widget)
        dock1D=Dock('Viewer1D')
        dock1D.addWidget(viewer1D_widget)


        #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        #% 2D viewer Dock
        viewer2D_widget=QtWidgets.QWidget()
        self.ui.viewer2D=Viewer2D(viewer2D_widget)
        dock2D=Dock('Viewer2D')
        dock2D.addWidget(viewer2D_widget)

        ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        ##% Navigator viewer Dock
        navigator1D_widget=QtWidgets.QWidget()
        self.ui.navigator1D=Viewer1D(navigator1D_widget)
        self.ui.navigator1D.ui.crosshair.crosshair_dragged.connect(self.update_viewer_data)
        self.ui.navigator1D.ui.crosshair_pb.click()
        navigator2D_widget=QtWidgets.QWidget()
        self.ui.navigator2D=Viewer2D(navigator2D_widget)
        self.ui.navigator2D.crosshair_dragged.connect(self.update_viewer_data) #export scaled position in conjonction with 2D scaled axes
        self.ui.navigator2D.ui.crosshair_pb.click()
        dock_nav=Dock('Navigator')
        dock_nav.addWidget(navigator1D_widget)
        dock_nav.addWidget(navigator2D_widget,1,0)

        self.dockarea.addDock(file_dock,'top')
        self.dockarea.addDock(dock2D,'right',file_dock)
        self.dockarea.addDock(dock1D,'bottom',dock2D)
        self.dockarea.addDock(dock_nav,'right',dock2D)

    def update_status(self,txt,wait_time=1000,log=''):
        """
            | Update the statut bar showing a Message with a delay of wait_time ms (1s by default)

            ================ ======== ===========================
            **Parameters**   **Type**     **Description**

             *txt*            string   the text message to show

             *wait_time*      int      the delay time of showing
            ================ ======== ===========================

        """
        self.ui.statusbar.showMessage(txt,wait_time)
        if log=='log':
            self.log_signal.emit(txt)


    def Quit_fun(self):
        """
            |
            | close the current instance of DAQ_Analysis

        """
        if self.h5file is not None:
            if self.h5file.isopen:
                self.h5file.close()
        self.mainwindow.close()

    def create_menu(self,menubar):
        """
             Set the filemenu structure with three elements splited by a separator at the 2nd position :
                * *open DAQ_scan file* : calling the do_load intern method
                * *close h5 file*      : calling the close_h5 intern method
                * *Quit*               : calling the quit_fun intern method

            ================ =============================================== ====================================
             **Parameters**           **Type**                                      **Description**

             *menubar*        instance of pyqtgraph.DockArea menuBar object   the generic menuBar object of menu
            ================ =============================================== ====================================

            See Also
            --------
            do_load, close_h5, quit_fun
        """
        file_menu=menubar.addMenu('File')
        open_action=file_menu.addAction("Open DAQ_Scan file")
        open_action.triggered.connect(self.do_load)
        close_action=file_menu.addAction("close h5 file")
        close_action.triggered.connect(self.close_h5)
        file_menu.addSeparator()
        quit_action=file_menu.addAction("Quit")
        quit_action.triggered.connect(self.Quit_fun)

    def close_h5(self):
        """
            | close the loaded h5 file if exists with clearing the ui-h5 tree structure

        """
        if self.h5file is not None:
            if self.h5file.isopen:
                self.h5file.close()
        self.ui.h5file_tree.ui.Tree.clear()

#should be one function from here
    def do_load(self,check_state):
        """
            | Call the loading h5 file procedure

            ================ ========= =============================
            **Parameters**   **Type**      **Description**

            *check_state*     none       not used
            ================ ========= =============================

            See Also
            --------
            load_h5_file
        """
        self.load_h5_file()

    def load_h5_file(self,path=None):
        """
            | Load the specific h5 file calling the open_h5_file procedure

            ================ ============ =======================================
            **Parameters**    **Type**            **Description**

             *path*           string        the current path to the file to load
            ================ ============ =======================================

            See Also
            --------
            open_h5_file, update_status, daq_utils.select_file
        """
        try:
            filename=select_file(start_path=path,save=False,ext='h5')
            if filename is not "":
                self.open_h5_file(filename)
        except Exception as e:
            self.update_status(str(e),wait_time=self.wait_time)

    def read_external_h5file(self,h5file):
        try:
            self.h5file=h5file
            self.populate_tree()
        except Exception as e:
            self.update_status(str(e),wait_time=self.wait_time)

    def open_h5_file(self,filename):
        """
            | Store the h5 file from the tables attributes and populate tree with raw h5 data

            =============== ========== =============================
            **Parameters**   **Type**         **Description**

             *filename*      string     name of the h5 file to load
            =============== ========== =============================

            See Also
            --------
            populate_tree, update_status
        """
        try:
            self.h5file=tables.open_file(str(filename))
            self.populate_tree()
        except Exception as e:
            self.update_status(str(e),wait_time=self.wait_time)
#to here (see do_load.py)

    def populate_tree(self):
        """
            | Init the ui-tree and store data into calling the h5_tree_to_Qtree convertor method

            See Also
            --------
            h5tree_to_QTree, update_status
        """
        try:
            if self.h5file is not None:
                self.ui.h5file_tree.ui.Tree.clear()
                base_node=self.h5file.root
                base_tree_item,pixmap_items=utils.h5tree_to_QTree(self.h5file,base_node)
                self.ui.h5file_tree.ui.Tree.addTopLevelItem(base_tree_item)
                self.add_widget_totree(pixmap_items)

                
        except Exception as e:
            self.update_status(str(e),wait_time=self.wait_time)

    def add_widget_totree(self,pixmap_items):
        
        for item in pixmap_items:
            widget=QtWidgets.QWidget()

            vLayout=QtWidgets.QVBoxLayout()
            label1D=QtWidgets.QLabel()
            bytes=QByteArray(item['node']._v_attrs['pixmap1D'])
            im1=QtGui.QImage.fromData(bytes)
            a=QtGui.QPixmap.fromImage(im1)
            label1D.setPixmap(a)

            label2D=QtWidgets.QLabel()
            bytes=QByteArray(item['node']._v_attrs['pixmap2D'])
            im2=QtGui.QImage.fromData(bytes)
            b=QtGui.QPixmap.fromImage(im2)
            label2D.setPixmap(b)

            vLayout.addWidget(label1D)
            VLayout.addwidget(label2D)
            widget.setLayout(vLayout)
            self.ui.h5file_tree.ui.Tree.setItemWidget(item['item'],1,widget)


    def show_h5_attributes(self,item,col):
        """
            | Show the h5 attribute using the intern tree structure and update settings and User Interface custom tree

            ================= ====================== ====================================
            **Parameters**      **Type**              **Description**

             *item*           tables Group instance   contain the root node of the tree

             *col*                                    not used
            ================= ====================== ====================================

            See Also
            --------
            daq_utils.custom_parameter_tree.XML_string_to_parameter, update_status
        """
        try:
            node=self.h5file.get_node(item.text(2))
            attrs=node._v_attrs
            attrs_names=attrs._f_list('all')
            attr_dict=OrderedDict([])
            for attr in attrs_names:
                if attr!='settings':
                    attr_dict[attr]=attrs[attr]
            self.ui.attributes_table.set_table_value(attr_dict)

            if hasattr(attrs,'settings'):
                for child in self.settings.children():
                    child.remove()
                QtWidgets.QApplication.processEvents() #so that the tree associated with settings updates
                params=custom_tree.XML_string_to_parameter(attrs.settings.decode())
                self.settings.addChildren(params)
                if hasattr(attrs,'scan_settings'):
                    params=custom_tree.XML_string_to_parameter(attrs.scan_settings.decode())
                    self.settings.addChildren(params)
        except Exception as e:
            self.update_status(str(e),wait_time=self.wait_time)


    def show_h5_data(self,item,col):
        """
             Show the h5 data reading in the tree structure with update (if needed) of :
                * viewer 1D
                * viewer 2D
                * navigator 1D
                * navigator 2D

            =============== ====================== ===================================
            **Parameters**   **Type**               **Description**

             *item*          tables Group instance  contain the root node of the tree
             *col*                                  not used
            =============== ====================== ===================================

            See Also
            --------
            show_h5_attributes, set_axis, get_scan_parent_node, get_scan_parent_node,  update_viewer_data, update_status
        """
        try:
            self.data_buffer=[]
            self.show_h5_attributes(item,col)
            node=self.h5file.get_node(item.text(2))
            attrs=node._v_attrs
            attrs_names=attrs._f_list('all')
            if attrs.CLASS=='ARRAY' or attrs.CLASS=='CARRAY': #means there is data to be plotted
                if 'scan_type' not in attrs_names: #means one of the navigation axis or other type of data
                    self.loaded_data=node.read()
                    self.loaded_data_scan_type=None
                    shape=self.loaded_data.shape
                    if len(shape)==1:
                        self.ui.viewer1D.remove_plots()
                        self.ui.viewer1D.x_axis=None
                        self.ui.viewer1D.show_data([self.loaded_data])
                    elif len(shape)==2:
                        #self.ui.viewer2D.remove_plots()
                        self.ui.viewer2D.setImage(self.loaded_data)

                elif attrs.scan_type=='Scan1D':
                    self.loaded_data_scan_type=attrs.scan_type
                    scan_node=self.get_scan_parent_node(node)
                    if scan_node is None:
                        return
                    self.ui.navigator1D.parent.setVisible(True)
                    self.ui.navigator2D.parent.setVisible(False)
                    self.loaded_data=node.read()
                    shape=self.loaded_data.shape
                    self.nav_x_axis=self.set_axis(scan_node,'scan_x_axis_unique',self.loaded_data.shape[0])
                    self.nav_y_axis=[0]
                    self.ui.navigator1D.remove_plots()
                    self.ui.navigator1D.x_axis=self.nav_x_axis
                    if attrs.data_type=='0D':
                        self.ui.navigator1D.show_data([self.loaded_data])
                    elif attrs.data_type=='1D':
                        self.x_axis=self.set_axis(node._v_parent,'x_axis',self.loaded_data.shape[1])
                        self.ui.navigator1D.show_data([np.sum(self.loaded_data,axis=1)])
                    elif attrs.data_type=='2D':
                        self.x_axis=self.set_axis(node._v_parent,'x_axis',self.loaded_data.shape[2])
                        self.y_axis=self.set_axis(node._v_parent,'y_axis',self.loaded_data.shape[1])
                        self.ui.navigator1D.show_data([np.sum(self.loaded_data,axis=(1,2))])
                    else:
                        pass

                elif attrs.scan_type=='Scan2D':
                    self.loaded_data_scan_type=attrs.scan_type
                    scan_node=self.get_scan_parent_node(node)
                    if scan_node is None:
                        return
                    self.ui.navigator1D.parent.setVisible(False)
                    self.ui.navigator2D.parent.setVisible(True)
                    self.loaded_data=node.read()
                    self.nav_x_axis=self.set_axis(scan_node,'scan_x_axis_unique',self.loaded_data.shape[0])
                    self.nav_y_axis=self.set_axis(scan_node,'scan_x_axis_unique',self.loaded_data.shape[1])
                    self.ui.navigator2D.set_scaling_axes(
                        scaling_options=edict(scaled_xaxis=edict(label="x axis",units=None,offset=np.min(self.nav_x_axis),scaling=self.nav_x_axis[1]-self.nav_x_axis[0]),
                                            scaled_yaxis=edict(label="y axis",units=None,offset=np.min(self.nav_y_axis),scaling=self.nav_y_axis[1]-self.nav_y_axis[0])))
                    if attrs.data_type=='0D':
                        self.ui.navigator2D.setImage(self.loaded_data)
                    elif attrs.data_type=='1D':
                        self.x_axis=self.set_axis(node._v_parent,'x_axis',self.loaded_data.shape[2])
                        self.ui.navigator2D.setImage(np.sum(self.loaded_data,axis=2))
                    elif attrs.data_type=='2D':
                        self.x_axis=self.set_axis(node._v_parent,'x_axis',self.loaded_data.shape[3])
                        self.y_axis=self.set_axis(node._v_parent,'y_axis',self.loaded_data.shape[2])
                        self.ui.navigator2D.setImage(np.sum(self.loaded_data,axis=(2,3)))
                    else:
                        pass
                self.update_viewer_data(self.nav_x_axis[0],self.nav_y_axis[0])


            else:
                return
        except Exception as e:
            self.update_status(str(e),wait_time=self.wait_time)

    def update_viewer_data(self,posx=0,posy=0):
        """
            | Update the viewer informations from an x/y given position and store data.
            | Ruled by the viewer type (0D,1D,2D)

            ================ ========= ==============================
            **Parameters**   **Type**        **Description**

             *posx*           int       the x position of the viewer

             *posy*           int       the y position of the viewer
            ================ ========= ==============================

            See Also
            --------
            update_status
        """
        try:
            if self.loaded_data is not None:
                shape=self.loaded_data
                scan_type=self.loaded_data_scan_type
                if scan_type=='Scan1D':
                    ind_x=utils.find_index(self.nav_x_axis,posx)[0][0]
                    data=self.loaded_data[ind_x]
                elif scan_type=='Scan2D':
                    ind_x=utils.find_index(self.nav_x_axis,posx)[0][0]
                    ind_y=utils.find_index(self.nav_y_axis,posy)[0][0]
                    data=self.loaded_data[ind_y,ind_x]

                if len(data.shape)==0: #means 0D data, plot on 1D viewer
                    self.data_buffer.append(data)
                    self.ui.viewer1D.show_data([np.array(self.data_buffer)])
                elif len(data.shape)==1: #means 1D data, plot on 1D viewer
                    self.ui.viewer1D.remove_plots()
                    self.ui.viewer1D.x_axis=self.x_axis
                    self.ui.viewer1D.show_data([data])
                elif len(data.shape)==2: #means 2D data, plot on 2D viewer
                    self.ui.viewer2D.set_scaling_axes(
                            scaling_options=edict(scaled_xaxis=edict(label="x axis",units=None,offset=np.min(self.x_axis),scaling=self.x_axis[1]-self.x_axis[0]),
                                                scaled_yaxis=edict(label="y axis",units=None,offset=np.min(self.y_axis),scaling=self.y_axis[1]-self.y_axis[0])))
                    self.ui.viewer2D.setImage(data)
        except Exception as e:
            self.update_status(str(e),wait_time=self.wait_time)


    def set_axis(self,node,axis_type,Npts):
        """
            | Set axis values from node and a linspace regular distribution

            ================ ======================= ==========================================
            **Parameters**     **Type**                **Description**

             *node*           tables Group instance   the root node of the local treated tree
            ================ ======================= ==========================================

            Returns
            -------
            float array
                the computed values axis.

        """
        try:
            axis=node._f_get_child(axis_type).read()# try to load some data relative to the axis
        except:
            axis=np.linspace(0,Npts,Npts,endpoint=False)

        return axis

    def get_scan_parent_node(self,node):
        """
            | Get the root level node from the given node (if exists)

            ================ ===================== ==========================================
            **Parameters**    **Type**              **Description**

             *node*           tables Node instance   the root node of the local treated tree
            ================ ===================== ==========================================

            Returns
            -------
            Node instance
                root level node.
        """
        while 'Scan' not in node._v_name:
            node=node._v_parent
            if node._v_name=='/': #reached root level
                node=None
                break
        return node

if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    win = QtWidgets.QMainWindow();
    area = DockArea()
    win.setCentralWidget(area)
    win.resize(1000,500)
    prog = DAQ_Analysis(area,title='DAQ_Analysis')
    win.show()
    prog.open_h5_file('C:\\Data\\2018\\20180212\\Dataset_20180212_000\\Dataset_20180212_000.h5')
    sys.exit(app.exec_())


from PyQt5 import QtGui, QtWidgets
from PyQt5.QtCore import Qt,QObject, pyqtSlot, QThread, pyqtSignal, QLocale, QDateTime, QSize
import sys
import pyqtgraph as pg
import numpy as np
from pymodaq.daq_utils import daq_utils as utils



class Viewer1DBasic(QtWidgets.QWidget,QObject):
    """this plots 1D data on a plotwidget. one linear region to select data
    """
    def __init__(self,parent=None):
        QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
        super(Viewer1DBasic, self).__init__()
        if parent is None:
            parent = QtWidgets.QWidget()
            parent.show()

        self.parent = parent

        self.plot_colors = utils.plot_colors

        self.data_to_export = None
        self.datas = None
        self._x_axis = None

        self.labels = []
        self.plot_channels = None
        self.legend = None
        self.setup_ui()

    def setup_ui(self):
        vboxlayout = QtWidgets.QVBoxLayout()
        self.plotwidget = pg.PlotWidget()
        self.parent.setLayout(vboxlayout)
        vboxlayout.addWidget(self.plotwidget)

        self.legend = self.plotwidget.addLegend()

    @pyqtSlot(list)
    def show_data(self, datas):
        if datas is not None:
            self.datas = datas

            if self.labels == [] or len(self.labels) != len(datas):
                self._labels = ["CH{}".format(ind) for ind in range(len(datas))]

            if self.plot_channels == None: #initialize data and plots
                self.ini_data_plots(len(datas))
            elif len(self.plot_channels) != len(datas):
                self.remove_plots()
                self.ini_data_plots(len(datas))

            for ind_plot, data in enumerate(datas):
                if self._x_axis is None or len(self._x_axis) != len(data):
                    self._x_axis = np.linspace(0,len(data), len(data), endpoint=False)

                self.plot_channels[ind_plot].setData(x=self._x_axis, y=data)

    def ini_data_plots(self, Nplots):
        self.plot_channels=[]

        channels = []
        for ind in range(Nplots):
            channel=self.plotwidget.plot()
            channel.setPen(self.plot_colors[ind])
            self.legend.addItem(channel, self._labels[ind])
            channels.append(ind)
            self.plot_channels.append(channel)

    def remove_plots(self):
        if self.plot_channels is not None:
            for channel in self.plot_channels:
                self.plotwidget.removeItem(channel)
            self.plot_channels = None
        if self.legend is not None:
            items = [item[1].text for item in self.legend.items]
            for item in items:
                self.legend.removeItem(item)

    def set_axis_label(self,axis_settings=dict(orientation='bottom', label='x axis', units='pxls')):
        axis = self.plotwidget.plotItem.getAxis(axis_settings['orientation'])
        axis.setLabel(text=axis_settings['label'], units=axis_settings['units'])

    @property
    def x_axis(self):
        return self._x_axis

    @x_axis.setter
    def x_axis(self, x_axis):
        label = 'Pxls'
        units = ''
        if isinstance(x_axis, dict):
            if 'data' in x_axis:
                xdata = x_axis['data']
            if 'label' in x_axis:
                label = x_axis['label']
            if 'units' in x_axis:
                units = x_axis['units']
        else:
            xdata = x_axis
        self._x_axis = xdata
        self.show_data(self.datas)
        self.set_axis_label(dict(orientation='bottom',label=label,units=units))



if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    Form=QtWidgets.QWidget()
    prog = Viewer1DBasic(Form)

    from pymodaq.daq_utils.daq_utils import gauss1D
    x=np.linspace(0,200,201)
    y1=gauss1D(x,75,25)
    y2=gauss1D(x,120,50,2)
    tau_half = 27
    tau2=100
    x0=50
    dx=20
    ydata_expodec = np.zeros((len(x)))
    ydata_expodec[:50] = 1*gauss1D(x[:50],x0,dx,2)
    ydata_expodec[50:] = 1*np.exp(-(x[50:]-x0)/(tau_half/np.log(2)))#+1*np.exp(-(x[50:]-x0)/tau2)
    ydata_expodec += 0.1*np.random.rand(len(x))

    prog.show_data([y1, y2, ydata_expodec])
    Form.show()
    prog.x_axis
    sys.exit(app.exec_())


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'viewer1D_GUI_dock.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(929, 769)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.splitter_2 = QtWidgets.QSplitter(Form)
        self.splitter_2.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_2.setObjectName("splitter_2")
        self.splitter = QtWidgets.QSplitter(self.splitter_2)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.horizontalLayoutWidget = QtWidgets.QWidget(self.splitter)
        self.horizontalLayoutWidget.setObjectName("horizontalLayoutWidget")
        self.horizontalLayout_settings = QtWidgets.QHBoxLayout(self.horizontalLayoutWidget)
        self.horizontalLayout_settings.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_settings.setObjectName("horizontalLayout_settings")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.zoom_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.zoom_pb.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Zoom_to_Selection.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.zoom_pb.setIcon(icon)
        self.zoom_pb.setCheckable(True)
        self.zoom_pb.setObjectName("zoom_pb")
        self.horizontalLayout.addWidget(self.zoom_pb)
        self.Do_math_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.Do_math_pb.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Calculator.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.Do_math_pb.setIcon(icon1)
        self.Do_math_pb.setCheckable(True)
        self.Do_math_pb.setObjectName("Do_math_pb")
        self.horizontalLayout.addWidget(self.Do_math_pb)
        self.do_measurements_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.do_measurements_pb.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/MeasurementStudio_32.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.do_measurements_pb.setIcon(icon2)
        self.do_measurements_pb.setCheckable(True)
        self.do_measurements_pb.setObjectName("do_measurements_pb")
        self.horizontalLayout.addWidget(self.do_measurements_pb)
        self.crosshair_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.crosshair_pb.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/reset.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.crosshair_pb.setIcon(icon3)
        self.crosshair_pb.setCheckable(True)
        self.crosshair_pb.setObjectName("crosshair_pb")
        self.horizontalLayout.addWidget(self.crosshair_pb)
        self.aspect_ratio_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.aspect_ratio_pb.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/zoomReset.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.aspect_ratio_pb.setIcon(icon4)
        self.aspect_ratio_pb.setCheckable(True)
        self.aspect_ratio_pb.setObjectName("aspect_ratio_pb")
        self.horizontalLayout.addWidget(self.aspect_ratio_pb)
        self.x_label = QtWidgets.QLabel(self.horizontalLayoutWidget)
        self.x_label.setObjectName("x_label")
        self.horizontalLayout.addWidget(self.x_label)
        self.y_label = QtWidgets.QLabel(self.horizontalLayoutWidget)
        self.y_label.setObjectName("y_label")
        self.horizontalLayout.addWidget(self.y_label)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_settings.addLayout(self.verticalLayout)
        self.Graph_Lineouts = PlotWidget(self.splitter)
        self.Graph_Lineouts.setObjectName("Graph_Lineouts")
        self.ROIs_widget = QtWidgets.QWidget(self.splitter_2)
        self.ROIs_widget.setMaximumSize(QtCore.QSize(300, 16777215))
        self.ROIs_widget.setObjectName("ROIs_widget")
        self.gridLayout.addWidget(self.splitter_2, 0, 0, 1, 1)
        self.StatusBarLayout = QtWidgets.QHBoxLayout()
        self.StatusBarLayout.setObjectName("StatusBarLayout")
        self.gridLayout.addLayout(self.StatusBarLayout, 1, 0, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.zoom_pb.setToolTip(_translate("Form", "Zoom View"))
        self.Do_math_pb.setToolTip(_translate("Form", "Do Math"))
        self.do_measurements_pb.setToolTip(_translate("Form", "Do advanced measurrments"))
        self.crosshair_pb.setToolTip(_translate("Form", "Show/hide crosshair"))
        self.aspect_ratio_pb.setToolTip(_translate("Form", "Show/hide crosshair"))
        self.x_label.setText(_translate("Form", "x:"))
        self.y_label.setText(_translate("Form", "y:"))

from pyqtgraph import PlotWidget
from pymodaq.QtDesigner_Ressources import QtDesigner_ressources_rc

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import QObject, pyqtSlot, QThread, pyqtSignal
import sys
from PyMoDAQ.DAQ_Utils.plotting.trace1D_selection_GUI import Ui_Dialog
import python_lib as mylib
import pyqtgraph
import numpy as np


class Trace1D_selection(Ui_Dialog):
    def __init__(self,parent,file_path):
        super(Ui_Dialog,self).__init__()
        self.ui=Ui_Dialog()
        self.ui.setupUi(parent)
        self.wait_time=2000

        self.ui.statusbar=QtWidgets.QStatusBar(parent)
        self.ui.verticalLayout_2.addWidget(self.ui.statusbar)

        self.ui.plot_area=self.ui.graphicsView.plotItem
        self.ui.plot_area.enableAutoRange('xy', True)
        self.ui.curve_1D=self.ui.plot_area.plot(pen='w')
        
        self.file_path=file_path
        self.selected_data=Selection_data(file_path)

        #connect signals
        self.ui.buttonBox.accepted.connect(self.selected_data.accept)
        self.ui.buttonBox.rejected.connect(self.selected_data.reject)
        self.ui.retry_pb.clicked.connect(self.import_data)
        self.ui.data_name_edit.editingFinished.connect(self.update_data_name)


        if self.file_path!=None:
            self.import_data()
    
    def update_data_name(self):
        self.selected_data.data_name=self.ui.data_name_edit.text()
                    
    def update_status(self,status,wait_time=0):
        self.ui.statusbar.showMessage(status,wait_time)

    def import_data(self):
        try:
            delimiter_text=self.ui.delimiter_combo.currentText()
            header_lines=self.ui.header_sb.value()
            xcol=self.ui.col_x_sb.value()
            ycol=self.ui.col_y_sb.value()

            if delimiter_text=="tab":
                delimiter='\t'
            elif delimiter_text=="comma":
                delimiter=','
            elif delimiter_text=="space":
                delimiter=" "

            data = np.loadtxt(self.file_path, delimiter=delimiter, skiprows=header_lines)
            shape=data.shape
            if len(shape)<=2:
                if shape[0]>shape[1]:
                    self.xdata=data[:,xcol]
                    self.ydata=data[:,ycol]
                else:
                    self.xdata=data[xcol,:]
                    self.ydata=data[ycol,:]
                self.ui.curve_1D.setData(self.xdata,self.ydata)
                self.selected_data.xdata=self.xdata
                self.selected_data.ydata=self.ydata
        except Exception as e:
            self.update_status(str(e),wait_time=self.wait_time)



class Selection_data(QObject):
    accept_reject=pyqtSignal(list)
    def __init__(self,file_path,xdata=None,ydata=None,data_name=None):
        super(QObject,self).__init__()
        self.xdata=xdata
        self.ydata=ydata
        self.data_name=data_name

    def accept(self):
        self.accept_reject.emit([True,self.data_name,self.xdata,self.ydata])
        
    def reject(self):
        self.accept_reject.emit([False,self.data_name,self.xdata,self.ydata])


if __name__ == '__main__':
	
    app = QtWidgets.QApplication(sys.argv);
    Dialog = QtWidgets.QDialog();
    file_path="D:\\Data\\2016\\Vincent\\trace_1D_test.dat";
    prog = Trace1D_selection(Dialog,file_path);
    Dialog.show();
    sys.exit(app.exec_())







